<?php
error_reporting(E_ALL & ~E_NOTICE & ~E_WARNING & ~E_STRICT);
// error_reporting(E_ALL ^ E_NOTICE ^ E_WARNING);
// error_reporting(E_ALL ^ E_NOTICE);

ini_set("session.gc_maxlifetime",84600);
@session_start();


class zstore {

	/*** Your options start here ***/
	// options are set in z-conf.php now
	/*** Your options end here ***/

	var $p = 1;

	var $configId = 1;

	var $error;
	var $success;
	var $info;
	var $json;


  var $source_descrs = array("admin-import" => "imported by admin",
															"amazon-suggestions" => "suggestions by amazon",
															"search-results" => "additional search results",
															"as_variation" => "variation - child from parent",
															"variation_parent" => "variation - parent from child");

	//for cart issues
	var $cartItemAdded = false;
	var $cartError = null;

	function __construct($setup=false) {

		//setting up config from external file
		include(dirname(dirname(__FILE__)) . "/z-conf.php");
		foreach ($_config as $key => $val) {
			$this->{$key} = $val;
		}

		$this->setup = $setup;

		if (!$setup) {
			//you can remove the next lines after setup has been finished from here [+]
			if (file_exists("setup.php")) {
				header("location: setup.php");
			}
			//.. to here - you can remove the previous lines after setup has been finished [-]
		}
    
		$this->dbconnect();

		$this->xmlCacheDir = dirname(dirname(__FILE__)) . "/_xmlCache/";

		//url
		$this->instDir = dirname($_SERVER["SCRIPT_NAME"]);
		if ($this->instDir == "/") $this->instDir = null;
		if (isset($_SERVER["HTTP_HOST"])) {
			$this->projectUrl = (isset($_SERVER["HTTPS"]) && ($_SERVER["HTTPS"] || $_SERVER["HTTPS"] === "on") ? "https" : "http") . "://" . $_SERVER["HTTP_HOST"];
		} else {
			$this->projectUrl = null;
		}
		$this->instUrl = $this->projectUrl . $this->instDir;

		$this->auth = isset($_SESSION["z-auth"]) ? $_SESSION["z-auth"] : false;

		// $this->lang = "en";

    if (isset($_REQUEST["p"]) && intval($_REQUEST["p"])) $this->p = abs($_REQUEST["p"]);    

		$this->setConfig();
		$this->setSite();

		$this->sendLastPHPError();

	}



  //connect to database - they mysqli way
  function dbconnect() {
	  $this->db = mysqli_connect($this->dbHost, $this->dbUser, $this->dbPass) or die("<b>Database-connection failed</b>,<br>please use the correct credentials and host data for your database.");
	  mysqli_set_charset($this->db,'utf8');
	  mysqli_select_db($this->db,$this->dbName) or die("Database <b>" . $this->dbName . " not found</b> or no rights for access.");
  }

	/**
	 * This functions is used to handle all database queries of the project
	 * and if an error occurs an usefull email to an administrator will be sent
	 * 
	 * @param string $sql - the SQL Query
	 * 
	 * @return the MYSQL result identifier, send's an email on error
	 **/ 
	function dbquery($sql,$quiet = false) {
	  //own db debugger

	  $res = mysqli_query($this->db,$sql);
	  if (mysqli_errno($this->db) && !$quiet) {
	  // if (mysqli_errno($this->db)) {
	    mail($this->adminMail, strip_tags($this->projectName) . ' DB-Error',$this->projectUrl . $_SERVER['PHP_SELF']."?".$_SERVER['QUERY_STRING']."\n\nMySQL error ".mysqli_errno($this->db).": ".mysqli_error($this->db)."\nWhen executing:\n$sql\nHTTP_USER_AGENT: ".$_SERVER['HTTP_USER_AGENT']."\nREMOTE_ADDR: ".$_SERVER['REMOTE_ADDR']."\nHTTP_HOST: ".$_SERVER["HTTP_HOST"] . $_SERVER["REQUEST_URI"]);
	  }
	  return $res;
	}


	/**
	 * quotes and escape quotes of single strings, or returns NULL for the use in mysql queries
	 * makes only sense for strings 
	 * Attention: the Quotes are part of the output:
	 *    dbquote("hallo")    ->    "'hallo'"
	 *    dbquote("c'mon")    ->    "'c\'mon'"
	 *    dbquote("")         ->    "NULL"  
	 *    use
	 *      field = " . dbquote($val) . "
	 *    don't use 
	 *      field = '" . dbquote($val) . "' 
	 * 
	 * @param string $str - the string to check
	 * 
	 * @return formated string for the use in sql statements
	 **/ 
	function dbquote($str) {
	  //to get the correct quotes and prevent from musthave of magic quotes
	  if (get_magic_quotes_gpc()) { //if slashes added and magic quotes are enabled
	      $str = stripslashes($str);
	  }
		$str = "'" . mysqli_real_escape_string($this->db,$str) . "'";
	  return $str;
	}

	function highlightText($text,$fileExt="php") {
	    if ($fileExt == "php") {
	        ini_set("highlight.comment", "#008000");
	        ini_set("highlight.default", "#000000");
	        ini_set("highlight.html", "#808080");
	        ini_set("highlight.keyword", "#0000BB; font-weight: bold");
	        ini_set("highlight.string", "#DD0000");
	    } elseif ($fileExt == "html") {
	        ini_set("highlight.comment", "green");
	        ini_set("highlight.default", "#CC0000");
	        ini_set("highlight.html", "#000000");
	        ini_set("highlight.keyword", "black; font-weight: bold");
	        ini_set("highlight.string", "#0000FF");
	    } elseif ($fileExt == "shell") {
	        ini_set("highlight.comment", "#999");
	        ini_set("highlight.default", "#333");
	        ini_set("highlight.html", "#000000");
	        ini_set("highlight.keyword", "navy; font-weight: bold");
	        ini_set("highlight.string", "#0000FF");
	    }
	    // ...

	    $text = trim($text);
	    $text = highlight_string("<?php " . $text, true);  // highlight_string() requires opening PHP tag or otherwise it will not colorize the text
	    $text = trim($text);
	    $text = preg_replace("|^\\<code\\>\\<span style\\=\"color\\: #[a-fA-F0-9]{0,6}\"\\>|", "", $text, 1);  // remove prefix
	    $text = preg_replace("|\\</code\\>\$|", "", $text, 1);  // remove suffix 1
	    $text = trim($text);  // remove line breaks
	    $text = preg_replace("|\\</span\\>\$|", "", $text, 1);  // remove suffix 2
	    $text = trim($text);  // remove line breaks
	    $text = preg_replace("|^(\\<span style\\=\"color\\: #[a-fA-F0-9]{0,6}\"\\>)(&lt;\\?php&nbsp;)(.*?)(\\</span\\>)|", "\$1\$3\$4", $text);  // remove custom added "<?php "

	    return $text;
	}



	/**
	 * send notification of latest PHP errors if occur to find troubles with the installation more efficient
	 ***/
	function sendLastPHPError() {
		if (!$this->config->log_errors) return false;
		$error = error_get_last();
		if ($error) {
			$httpReferer = isset($_SERVER["HTTP_REFERER"]) ? $_SERVER["HTTP_REFERER"] : null;
			$url = $this->projectUrl . $_SERVER['PHP_SELF'] . (isset($_SERVER['QUERY_STRING']) ? "?" . $_SERVER['QUERY_STRING'] : "");
			$sql = "insert into `" . $this->tablePrefix . "error_log` set
				error_type = 'php',
				message = " . $this->dbquote($error["message"]) . ",
				file = " . $this->dbquote($error["file"]) . ",
				line = " . intval($error["line"]) . ",
				url = " . $this->dbquote($url) . ",
				referer = " . $this->dbquote($httpReferer) . ",
				ip = " . $this->dbquote(isset($_SERVER["REMOTE_ADDR"]) ? $_SERVER["REMOTE_ADDR"] : $_SERVER['SERVER_ADDR']) . ",
				created = now()";
			$this->dbquery($sql);
	    // mail($this->adminMail, strip_tags($this->projectName) . ' PHP-Site-Error', $this->projectUrl . $_SERVER['PHP_SELF']."?".$_SERVER['QUERY_STRING']."\n---\n" .print_r($error,true));
		}
	}


	/**
	* show saved errors
	 ***/
	function getErrorLogEntries() {
		$logEntries = array();
		//group entries to get groups :)
		$sql = "select count(id) as cnt, file, message, min(created) as latest from `" . $this->tablePrefix . "error_log` group by file, message order by latest desc";
		$res = $this->dbquery($sql);
		while ($row = mysqli_fetch_object($res)) {
			$row->relFile = preg_replace('!^'.realpath($_SERVER["DOCUMENT_ROOT"]).'!', "", str_replace('\\','/',$row->file));
			$logGroupKey = md5($row->file.$row->message);
			$logEntries[$logGroupKey] = $row;
		}
		if ($logEntries) {
			//all the log entries
			$sql = "select * from `" . $this->tablePrefix . "error_log` order by file, created desc";
			$res = $this->dbquery($sql);
			while ($row = mysqli_fetch_object($res)) {
				$logGroupKey = md5($row->file.$row->message);
				$row->relFile = preg_replace('!^'.realpath($_SERVER["DOCUMENT_ROOT"]).'!', "", str_replace('\\','/',$row->file));
				$logEntries[$logGroupKey]->logItems[] = $row;
			}		
		}
		return $logEntries;
	}


	/**
	* remove log entries
	 ***/
	function removeLogEntries($logGroup) {
		$logGroup = trim($logGroup);
		$error = $success = false;
		if (!$logGroup) {
			$error[] = "Insufficient Data sent.";
		} else {
			$sql = "delete from `" . $this->tablePrefix . "error_log` where md5(concat(file,message)) = " . $this->dbquote($logGroup);
			$this->dbquery($sql);
			$success = "<strong>" . intval(mysqli_affected_rows($this->db)) . "</strong> Log Entries were successfully deleted.";
		}
		$this->json = json_encode(array("success" => $success, "error" => $error));
	}

	/**
	* empty error_log
	 ***/
	function emptyErrorLog() {
		
		$sql = "delete from `" . $this->tablePrefix . "error_log`";
		$this->dbquery($sql);
		$success = "<strong>" . intval(mysqli_affected_rows($this->db)) . "</strong> Log Entries were successfully deleted.";
		$this->json = json_encode(array("success" => $success));
	}

	/**
	* enable disable error logging
	 ***/
	function updateErrorLogConfig($log_errors) {
		$log_errors = $log_errors ? 1 : 0;
		$error = $success = false;
		if (!$error) {
			$sql = "update `" . $this->tablePrefix . "config` set log_errors = '$log_errors' where id = '" . intval($this->configId) . "'";
			$this->dbquery($sql);
			$success = true;		

		}
 		$this->json = json_encode(array("success" => $success, "error" => $error
 			));
	}



	//authenticate
	function authenticate($data) {
		$error = array();
		$success = false;

		$user = trim($data["user"]);
		$pass = trim($data["pass"]);

		if (!$user) $error[] = "Please type in your username.";
		if (!$pass) $error[] = "Please type in your password.";

		if (!$error) {
			if ($user != $this->adminUser || $pass != $this->adminPass) {
				$error[] = "Username and password do not match.";
			} else {
		  	$this->auth = $_SESSION["z-auth"] = true;
			}
		}
 		$this->error = $error;
	}

	//logout
	function logout() {
		if ($this->auth) {
			$this->auth = $_SESSION["z-auth"] = false;
			$this->success = "You have been logged out successfully.";
			return true;
		} else {
			return false;
		}
	}



	/**
	 * Checks if email address is valid and host exists
	 * 
	 * @param string $email - the email to check
	 * 
	 * @return true if it's valid or false if not    
	 **/ 
	function is_email($email) {
	  $email = trim($email);
	  if (!$email) return false;
	  if (!preg_match("!^[\w|\.|\-|_]+@\w[\w|\.|\-]+\.[a-zA-Z]{2,}$!",$email)) return false;
	  else {
	    list($user, $host) = explode("@", $email);
	    if (!function_exists("checkdnsrr")) {
	    	return true;
	    } else {
		    if (checkdnsrr($host, "MX") or checkdnsrr($host, "A")) return true;
		    else return false;
	  	}
	  }
	}

	function is_assocArray(array $arr) {
    if (array() === $arr) return false;
    return array_keys($arr) !== range(0, count($arr) - 1);
	}

	/**
	 * converts and removes special characters from given file names
	 *  
	 * @param string $str - the name of the file
	 * 
	 * @return "special character"-less filename
	 **/
	function tidyup($str) {
	  $str = trim($str);
	  $str = strtolower($str);
	  $find = array(" ",".","ä","ö","ü","ß",",");
	  $repl = array("_","-","ae","oe","ue","ss","");
	  $str = str_replace($find,$repl,$str);
	  $str = preg_replace('([^a-z0-9\-_])', '', $str);
	  return $str;
	}


	//social sharing urls for current url - if second parameter isn't given
	function getSocialSharingURL($socialNetwork, $addToUrl = null, $url = null) {
		$sharingUrl = null;
		if (!$url) {
			$url = $this->projectUrl . $_SERVER["REQUEST_URI"];
		}

		switch ($socialNetwork) {
			case 'facebook':
				$sharingUrl = "https://www.facebook.com/sharer/sharer.php?u=" . urlencode($url);
				break;
			case 'twitter':
				$sharingUrl = "https://twitter.com/home?status=" . urlencode($url . ($addToUrl ? " " . $addToUrl : ""));
				break;
			case 'googlePlus':
				$sharingUrl = "https://plus.google.com/share?url=" . urlencode($url);
				break;
		}
		return $sharingUrl;
	}


/**
 * turns non compatible video urls into compatible ones
 * currently used for youtube and vimeo urls 
 * 
 * @param string $url - the URL to the video
 * 
 * @return converted url
 **/ 
function VURLembedded($url) {
    $url = trim($url);
    if (!$url) return false;
    //Youtube
    if (preg_match("/^http(s)?:\/\/(.*)youtu\.?be/i",$url)) {
      if (preg_match("/youtu\.be/",$url)) $vkey = basename($url);
      else {
        $q = parse_url($url,PHP_URL_QUERY);
        if (!$q) return $url;
        parse_str($q,$op);
        $vkey = $op["v"];
      }
      if (!$vkey) return false;      
      else return "https://www.youtube.com/v/$vkey?fs=1";
    }
    //Vimeo
    if (preg_match("/^http(s)?:\/\/(.*)vimeo/i",$url)) {
      $q = basename(parse_url($url,PHP_URL_PATH));
      if (!$q) return $url;
      if (is_numeric($q)) {
        return "https://vimeo.com/moogaloop.swf?clip_id=$q&amp;server=vimeo.com&amp;fullscreen=1";
      } else return $url;
    }
  }

/**
 * turns non compatible video urls into compatible ones for embedding with iframes
 * currently used for youtube and vimeo urls 
 * 
 * @param string $url - the URL to the video
 * 
 * @return converted url
 **/ 
  function vUrlIFrameEmbedded($url,$autoplay=0) {

    $url = trim($url);
    if (!$url) return false;
    if (preg_match('/www\.youtube\.com/',$url)) {
      //http://www.youtube.com/watch?v=gch6edum9Tg 
      $t = parse_url($url);
      $q = $t["query"];
      if (!$q) return false;
      parse_str($q,$op);
      if (!$op["v"]) return false;
      else $vurl = "https://www.youtube.com/embed/" . $op["v"] . "?rel=0&amp;autoplay=$autoplay";
    } elseif (preg_match('/youtu\.be/',$url)) {
      //http://youtu.be/gch6edum9Tg
      $t = parse_url($url);
      $q = basename($t["path"]);
      if (!$q) return false;
      else $vurl = "https://www.youtube.com/embed/$q?rel=0&amp;autoplay=$autoplay";

    } elseif (preg_match('/vimeo\.com/',$url)) {
      //http://vimeo.com/21864555
      $t = parse_url($url);
      $q = basename($t["path"]);
      if (!$q) return false;
      else $vurl = "https://player.vimeo.com/video/$q?autoplay=$autoplay";
    } else return false;
    if ($vurl) {
      return $vurl;
    }
  }


/**
 * retrieve video dimensions
 * currently used for youtube and vimeo urls 
 * 
 * @param string $url - the URL to the video
 * 
 * @return converted url
 **/ 
  function getVideoDimensions($url,$maxwidth=480) {
    $defaultW = 480;
    $defaultH = 270;
    $url = trim($url);
    if (preg_match('/(www\.youtube\.com)|(youtu\.be)/',$url)) {

      //CURL Version
      $XMLurl ="https://www.youtube.com/oembed?url=" . urlencode($url) . "&format=xml";
      $ch = curl_init();
      curl_setopt($ch, CURLOPT_URL, $XMLurl);
      curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
      // get the result of http query
      $output = curl_exec($ch);
      curl_close($ch);
      
      if (!preg_match('/^</',$output)) {
        if ($this->auth) echo "<div class='alert alert-info'>Video data of $url not completed loaded.<br />
          Output:<pre>$output</pre></div>";
      } else {
        $xmlData = simplexml_load_string($output);
      }


      //url_fopen Version
      // $xmlData = simplexml_load_file("https://www.youtube.com/oembed?url=" . urlencode($url) . "&format=xml");

      // echo "<pre>" .print_r($xmlData->width,true) . "</pre>";

    } elseif (preg_match('/vimeo\.com/',$url)) {
      
      //CURL Version
      $XMLurl ="https://vimeo.com/api/oembed.xml?url=" . urlencode($url) . "&format=xml";
      $ch = curl_init();
      curl_setopt($ch, CURLOPT_URL, $XMLurl);
      curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
      // get the result of http query
      $output = curl_exec($ch);
      curl_close($ch);
      
      if (!preg_match('/^</',$output)) {
        if ($this->auth) echo "<div class='alert alert-info'>Video data of $url not completed loaded.<br />
          Output:<pre>$output</pre></div>";
      } else {
        $xmlData = simplexml_load_string($output);
      }
      //url_fopen Version
      // $xmlData = simplexml_load_file("https://vimeo.com/api/oembed.xml?url=" . urlencode($url) . "&format=xml");

      // echo "<pre>" .print_r($xmlData->width,true) . "</pre>";
    }
    if ($xmlData->width && $xmlData->height) {
      $vw = $xmlData->width;
      $vh = $xmlData->height;
     } else {
      $vw = $defaultW;
      $vh = $defaultH;
     }
     //recalc if necessary
     if ($vw > $maxwidth) {
      $vh = ceil(($maxwidth / $vw) * $vh);
      $vw = $maxwidth;
     }

    //Ratio calculator
     $quotient = $vw/$vh;
     if (round($quotient) == 1) $ratio = "1by1";
     elseif (round($quotient*3) == 4) $ratio = "4by3";
     elseif (round($quotient*9) == 16) $ratio = "16by9";
     elseif (round($quotient*9) == 21) $ratio = "21by9";

     if (!$ratio) $ratio = "16by9";

     return array($vw,$vh,$ratio);
  }


  function countItemsInCategories() {
  	$catItemsCount = $catFamilyItemsCount = array();
  	//items for each category
  	$sql = "select count(id) as cnt, category_id from `" . $this->tablePrefix . "category_products` group by category_id";
		$res = $this->dbquery($sql);
		while ($row = mysqli_fetch_object($res)) {
			$catItemsCount[$row->category_id] = $row->cnt;
		}
		//combine for category families
		$sql = "select id, parent from `" . $this->tablePrefix . "categories` order by parent";
		$res = $this->dbquery($sql);
		while ($row = mysqli_fetch_object($res)) {
			$catFamilyItemsCount[$row->parent ?: $row->id] += $catItemsCount[$row->id];
		}
		return array($catItemsCount,$catFamilyItemsCount);
  }


  //get sources where the items are came from
  function getItemSources() {
  	$sources = array();
		$sql = "select count(id) as cnt, source from `" . $this->tablePrefix . "products` group by source";
		$res = $this->dbquery($sql);
		while ($row = mysqli_fetch_object($res)) {
			$sources[] = $row;
		}
		return $sources;
  }




	//get only categories with assigned products for the front end 
	function getCategoriesWithItems() {
		$mainCategories = $subCategories = $allCategoriesData = array();
		$add2sql = null;
		//count items
		list($catItemsCount,$catFamilyItemsCount) = $this->countItemsInCategories();

		if (!$this->auth) $add2sql = "where status = 1";
		$sql = "select * from `" . $this->tablePrefix . "categories` $add2sql order by parent, pos";
		$res = $this->dbquery($sql);
		while ($row = mysqli_fetch_object($res)) {
			$row->itemsCount = intval($catItemsCount[$row->id]);
			$row->url = $this->instDir . "/cat" . $row->id . "/" . $this->tidyup($row->descr) . ".html";
			if (!$row->parent) {
				if (!$catFamilyItemsCount[$row->id]) {
					continue;
				}
				$mainCategories[$row->id] = $row;
			} else {
				if (!$catItemsCount[$row->id]) {
					continue;
				}
				$subCategories[$row->parent][] = $row;
			}
			$allCategoriesData[$row->id] = $row;
		}
		return array($mainCategories,$subCategories,$allCategoriesData);
	}

	//get ids for children of categories
	function getCategoryChildrenIds($parent) {
		$parent = intval($parent);
		$catIds = array();
		$sql = "select id from `" . $this->tablePrefix . "categories` where parent = '$parent'";
		$res = $this->dbquery($sql);
		while ($row = mysqli_fetch_object($res)) {
			$catIds[] = $row->id;
		}
		return $catIds;
	}

	//get random first images for the category listings including childs
	function getRandomItemPicsForCategory($catId) {
		$catId = intval($catId);
		$images = array();
		$catIds = $this->getCategoryChildrenIds($catId);
		$catIds[] = $catId;
		if ($catIds) {
			$sql = "select id, asin, image_url from `" . $this->tablePrefix . "products` where 
				imported is not NULL and 
				image_url is not NULL and image_url != '' and
				id in (select product_id from `" . $this->tablePrefix . "category_products` where category_id in (" . implode(",",$catIds) . "))
				order by rand() limit 3";
			$res = $this->dbquery($sql);
			while ($row = mysqli_fetch_object($res)) {
				$images[] = $row;
			}
		}
		return $images;
	}	


	//get random products (for home on BNS) of parent categories and their children if needed
	function getRandomProducts($categoryId,$limit=4,$hasImages=false) {
		$categoryId = intval($categoryId);
		if (!$categoryId) return false;
		$items = array();

		$catIds = $this->getCategoryChildrenIds($categoryId);
		$catIds[] = $categoryId;

		$add2sql = null;
		if ($hasImages) {
			$add2sql = "and image_url != ''";
		}

		$sql = "select product_id from `" . $this->tablePrefix . "category_products` where 
			category_id in (" . implode(",",$catIds) . ") and 
			product_id in (select id from `" . $this->tablePrefix . "products` where imported is not null $add2sql)
			order by rand() limit " . intval($limit);
		$res = $this->dbquery($sql);
		while ($row = mysqli_fetch_object($res)) {
			$items[] = $this->getProductData(null,$row->product_id,"small");
		}
		return $items;
	}





	//get all categories mainly for admin purposes 
	function getCategoriesOld($withFamilyStructure=true) {
		$categories = $itemsCount = array();
		$add2sql = null;
		//count items
		$sql = "select count(id) as cnt, category_id from `" . $this->tablePrefix . "category_products` group by category_id";
		$res = $this->dbquery($sql);
		while ($row = mysqli_fetch_object($res)) {
			$itemsCount[$row->category_id] = $row->cnt;
		}


		if (!$this->auth) $add2sql = "where status = 1";
		$sql = "select * from `" . $this->tablePrefix . "categories` $add2sql order by parent, pos";
		$res = $this->dbquery($sql);
		while ($row = mysqli_fetch_object($res)) {
			$row->itemsCount = intval($itemsCount[$row->id]);
			$row->url = $this->instDir . "/cat" . $row->id . "/" . $this->tidyup($row->descr) . ".html";
			if ($withFamilyStructure) {
				if (!$row->parent) {
					$categories[$row->id]["parent"] = $row;
				} else {
					$categories[$row->parent]["children"][] = $row;
				}
			} else {
				$categories[] = $row;
			}
		}
		return $categories;
	}

	//get really all categories
	function getCategories() {
		$mainCategories = $subCategories = $allCategoriesData = $itemsCount = array();
		$noCatItemCount = 0;
		$add2sql = null;
		//count items
		$sql = "select count(id) as cnt, category_id from `" . $this->tablePrefix . "category_products` group by category_id";
		$res = $this->dbquery($sql);
		while ($row = mysqli_fetch_object($res)) {
			$itemsCount[$row->category_id] = $row->cnt;
		}

		//get itemnumber of non categorized items
		$sql = "select count(id) as cnt from `" . $this->tablePrefix . "products` where id not in (select product_id from `" . $this->tablePrefix . "category_products`)";
		$res = $this->dbquery($sql);
		while ($row = mysqli_fetch_object($res)) {
			$noCatItemCount = $row->cnt;
		}

		if (!$this->auth) $add2sql = "where status = 1";
		$sql = "select * from `" . $this->tablePrefix . "categories` $add2sql order by parent, pos";
		$res = $this->dbquery($sql);
		while ($row = mysqli_fetch_object($res)) {
			$row->itemsCount = intval($itemsCount[$row->id]);
			$row->url = $this->instDir . "/cat" . $row->id . "/" . $this->tidyup($row->descr) . ".html";
			$row->internal_parent_child_sort_coding = $row->parent ? $row->parent . "." . $row->id : $row->id;
			if (!$row->parent) {
				$mainCategories[$row->id] = $row;
			} else {
				$subCategories[$row->parent][] = $row;
			}
			$allCategoriesData[$row->id] = $row;
		}
		return array($mainCategories,$subCategories,$allCategoriesData,$noCatItemCount);
	}


	//get single category data 
	function getCategoryData($catId,$json=true) {
		$error = $catData = false;
		$catId = intval($catId);
		if(!$catId) $error = true;

		if (!$error) {
			$sql = "select * from `" . $this->tablePrefix . "categories` where id = '$catId'";
			$res = $this->dbquery($sql);
			if ($res && mysqli_num_rows($res)) {
				$catData = mysqli_fetch_object($res);
				$catData->url = $this->instDir . "/cat" . $catData->id . "/" . $this->tidyup($catData->descr) . ".html";
			}
		}

		if ($json) {
			$this->json = json_encode(array("success" => $success, "error" => $error, "catData" => $catData));
		} else {
			return $catData;
		}
	}

	//get all parent caegories 
	function getCategoryParents($parent = 0) {
		$parent = intval($parent);
		$parents = array();
		$sql = "select * from `" . $this->tablePrefix . "categories` where parent = '$parent'";
		$res = $this->dbquery($sql);
		while ($row = mysqli_fetch_object($res)) {
			$parents[] = $row;
		}
		$this->json = json_encode(array("parents" => $parents));
	}

	//remove single category data 
	function removeCategory($catId) {
		$error = false;
		$catId = intval($catId);
		$parent = 0;
		if(!$catId) $error = "Please select a category to delete";

		if (!$error) {
			$sql = "select * from `" . $this->tablePrefix . "categories` where id = '$catId'";
			$res = $this->dbquery($sql);
			if (!mysqli_num_rows($res)) $error = "The selected category cannot be found.";
			else {
				$row = mysqli_fetch_object($res);
				$parent = $row->parent;
			}
		}
		if (!$error) {
			//remove category
			$sql = "delete from `" . $this->tablePrefix . "categories` where id = '$catId'";
			$this->dbquery($sql);
			if (!$parent) {
				//remove assignments
				$sql = "delete from `" . $this->tablePrefix . "category_products` where category_id = '$catId'";
				$this->dbquery($sql);
			} else {
				//change assignments
				$sql = "update `" . $this->tablePrefix . "category_products` set category_id = '$parent' where category_id = '$catId'";
				$this->dbquery($sql);
			}
			$success = "The category has been deleted successfully.";
			$this->reposCategories($parent);
		}

		$this->json = json_encode(array("success" => $success, "error" => $error));

	}

	function getProducts(array $filter) {
		$products = $add2sql = $parentAsins = array();
		$orderbydirection = false;
		$limitSql = null;
		// $orderby = "lower(title)";
		$orderby = "if (title_custom is not null and title_custom != '',lower(title_custom),lower(title))";
		if (is_array($filter)) {
			if (isset($filter["srch"]) && $filter["srch"]) {
				$srch = strtolower(trim($filter["srch"]));
				//multi words???
				if (preg_match('/\s/',$srch)) {
					$srchWords = preg_split('/\s+/',$srch);
					$add2sqlsrchWords = array();
					foreach ($srchWords as $srchWord) {
						//OR combo
						$add2sqlsrchWords[] = "lower(title) like " . $this->dbquote("%$srchWord%") . " or
							lower(title_custom) like " . $this->dbquote("%$srchWord%") . " or
							lower(search_keywords) like " . $this->dbquote("%$srchWord%") . " or
							lower(actors) like " . $this->dbquote("%$srchWord%");
					}
					$add2sql[] = "(" . implode(" or ", $add2sqlsrchWords) . ")";
				} else {
					//including ASIN stringwise
					$add2sql[] = "(
						lower(title) like " . $this->dbquote("%$srch%") . " or
						lower(title_custom) like " . $this->dbquote("%$srch%") . " or
						lower(search_keywords) like " . $this->dbquote("%$srch%") . " or
						lower(actors) like " . $this->dbquote("%$srch%") . " or
						lower(asin) = " . $this->dbquote($srch) . "
						)";
				}
			}

			if (isset($filter["parentAsin"]) && $filter["parentAsin"]) {
				$add2sql[] = "(parent_asin = " . $this->dbquote($filter["parentAsin"]) . " or asin = " . $this->dbquote($filter["parentAsin"]) . ")";
			}

			if (isset($filter["own_cat"]) && $filter["own_cat"]) {
				$category_id = intval($filter["own_cat"]);
				if ($category_id > 0) {
					$add2sql[] = "id in (select product_id from `" . $this->tablePrefix . "category_products` where category_id = '$category_id')";
				} else {
					$add2sql[] = "id not in (select product_id from `" . $this->tablePrefix . "category_products`)";
				}
			}
			if (isset($filter["imported"]) && $filter["imported"]) {
				$add2sql[] = "imported is not null";
			}

			if (isset($filter["source"]) && $filter["source"]) {
				$add2sql[] = "source = " . $this->dbquote($filter["source"]);
			}

			if (isset($filter["noImg"]) && $filter["noImg"]) {
				$add2sql[] = "(image_url = '' or image_url is NULL)";
			}

			if (isset($filter["noTitle"]) && $filter["noTitle"]) {
				$add2sql[] = "(title = '' or title is NULL)";
			}

			if (isset($filter["desc"]) && $filter["desc"]) {
				$orderbydirection = "desc";
			}
			if (isset($filter["orderby"]) && $filter["orderby"]) {
				if ($filter["orderby"] == "created") {
					$orderby = "created";
				}
				if ($filter["orderby"] == "title") {
					// $orderby = "lower(title)";
					$orderby = "if (title_custom is not null and title_custom != '',lower(title_custom),lower(title))";
				}
			}

			//favorites
			if (array_key_exists("favAsins",$filter)) {
				$favAsins = $filter["favAsins"];
				if (!$favAsins) {
					$add2sql[] = "0";
				} else {
					$favAsins = array_map(
						function($val) {
							return preg_replace('/[^a-zA-Z0-9]/','',$val);
						}, 
						$favAsins
					);
					$add2sql[] = "asin in ('" . implode("','", $favAsins) . "')";
				}
			}
		}

		if (isset($filter["hasImages"]) && $filter["hasImages"]) {
			$add2sql[] = "image_url != ''";
		}
		


		//just to get totals ...
		$sql = "select id from `" . $this->tablePrefix . "products` " . ($add2sql ? "where " . implode(" and ",$add2sql) : "");
    $res = $this->dbquery($sql);
    $total = mysqli_num_rows($res);
		//.. for the pager
		if ($this->perPage) {
      if (($this->p * $this->perPage) >= ($total+$this->perPage)) $this->p = 1;       //if you are redirecting to page without ..., the page will be set to 1
      $offset = ($this->p * $this->perPage) - $this->perPage;
      $this->pages = ceil($total/$this->perPage);
      $limitSql = " limit $offset, " . $this->perPage;
    }

    if (isset($filter["display"]) && $filter["display"] == "all") {
    	$limitSql = null;
    }


		$sql = "select * from `" . $this->tablePrefix . "products` 
			" . ($add2sql ? "where " . implode(" and ",$add2sql) : "") . "
			order by $orderby $orderbydirection 
			$limitSql";
		$res = $this->dbquery($sql);
		while ($row = mysqli_fetch_object($res)) {

	  	$row->titleDB = $row->title_custom ?: $row->title;
			$row->url = $this->instDir . "/itm/" . $row->asin . "/" . $this->tidyup($row->titleDB) . ".html";

			$row->item_url = $this->placeholdersToAmzKeys($row->item_url);

			$row->is_child = $row->parent_asin && $row->parent_asin != $row->asin;

			$parentAsins[$row->asin] = $row->id;

			$products[$row->id] = $row;
		}

		if ($parentAsins) {
			//number of children
			$sql = "select count(id) as cnt, parent_asin from `" . $this->tablePrefix . "products` where parent_asin in ('" . implode("','",array_keys($parentAsins)) . "') and parent_asin != asin and parent_asin != ''";
			$res = $this->dbquery($sql);
			while ($row = mysqli_fetch_object($res)) {
				if (isset($products[$parentAsins[$row->parent_asin]])) {
					$products[$parentAsins[$row->parent_asin]]->numberOfChildren = $row->cnt;
				}
			}
		}


		//categories blabla
		if ($products) {
			$sql = "select * from `" . $this->tablePrefix . "category_products` where product_id in (" . implode(",",array_map("intval",array_keys($products))) . ")";
			$res = $this->dbquery($sql);
			while ($row = mysqli_fetch_object($res)) {
				// $products[$row->product_id]->categories[] = $row;				
				$products[$row->product_id]->catIds[] = $row->category_id;
			}			
		}



		return $products;
	}


	function getOtherGroupItems($productId,$limit=10) {
		$productId = intval($productId);
		if (!$productId) return false;
		$items = array();

		$sql = "select product_id from `" . $this->tablePrefix . "category_products` where 
			category_id in (select category_id from  `" . $this->tablePrefix . "category_products` where product_id = '$productId') and
			product_id != '$productId'
			order by rand()
			limit $limit";
		$res = $this->dbquery($sql);
		while ($row = mysqli_fetch_object($res)) {
			$items[] = $this->getProductData(null,$row->product_id,"small");
		}
		return $items;
	}



	function generateItemUrl($asin,$title) {
		$asin = trim($asin);
		$title = trim($title);
		if ($asin && $title) {
			return $this->instDir . "/itm/" . $asin . "/" . $this->tidyup($title) . ".html";
		}
	}



	//output - large/small ... default large - small would only return product data w/o reviews and other stuff

	function getProductData($asin,$id=0,$output="large") {
		$asin = trim($asin);
		$id = intval($id);
		$add2sql = array();
		if ($asin) {
			$add2sql[] = "asin = " . $this->dbquote($asin);
		} elseif ($id) {
			$add2sql[] = "id = '$id'";
		} 
		if (!$add2sql) {
			return false;
		}

		$sql = "select * from `" . $this->tablePrefix . "products` " . ($add2sql ? "where " . implode(" and ",$add2sql) : "");
		$res = $this->dbquery($sql);
		$product = mysqli_fetch_object($res);

		// if (!$product) {
		// 	//import on the fly....
		// 	$sql = "insert into `" . $this->tablePrefix . "products` set asin = " . $this->dbquote($asin) . ", created = now()";
		// 	$this->dbquery($sql);
		// 	if (!$this->amzFirstItemUpdate($row->asin)) {
		// 		$sql = "update `" . $this->tablePrefix . "products` set import_failed = now() where asin = " . $this->dbquote($asin);
		// 		$this->dbquery($sql);
		// 		return false;
		// 	} else {
		// 		$this->getProductData($asin,$id,$output);
		// 		return false;
		// 	}
		// 	// return false;
		// }

		if (!$product) return false;


		//let's initiate a quick import
		if (!$product->imported) return -1;

	  $product->titleDB = $product->title_custom ?: $product->title;
		$product->url = $this->instDir . "/itm/" . $product->asin . "/" . $this->tidyup($product->titleDB) . ".html";


		if ($output == "small") {
			return $product;
		}

		if ($product) {
			//assigned images
			$product->images = array();
			//add first image to array - if ther's any
			if ($product->image_url) {
				array_push($product->images, $product->image_url);
			}
			$sql = "select * from `" . $this->tablePrefix . "image_sets` where asin = " . $this->dbquote($product->asin) . " order by pos";
			$res = $this->dbquery($sql);
			while ($row = mysqli_fetch_object($res)) {
				$product->images[] = $row->image_url;
			}
			//remove dupes
			$product->images = array_unique($product->images);

			//assigned product information / reviews
			$product->editorial_reviews = array();
			$sql = "select * from `" . $this->tablePrefix . "editorial_reviews` where asin = " . $this->dbquote($product->asin) . " order by pos";
			$res = $this->dbquery($sql);
			while ($row = mysqli_fetch_object($res)) {
				$product->editorial_reviews[] = $row;
			}
			//personalize links
			$product->reviews_iframe_url = $this->placeholdersToAmzKeys($product->reviews_iframe_url);
			$product->item_url = $this->placeholdersToAmzKeys($product->item_url);

			//categories
			$product->categories = array();
			$sql = "select * from `" . $this->tablePrefix . "categories` where 
				id in (select category_id from `" . $this->tablePrefix . "category_products` where product_id = '" . $product->id . "') and
				status = 1";
			$res = $this->dbquery($sql);
			while ($row = mysqli_fetch_object($res)) {
				$row->url = $this->instDir . "/cat" . $row->id . "/" . $this->tidyup($row->descr) . ".html";
				$product->categories[] = $row;
			}

			// //variations with images and attribute values
			// $product->variations = array();
			// $sql = "select * from `" . $this->tablePrefix . "variations` where 
			// 	parent_asin = " . $this->dbquote($product->asin) . " or 
			// 	parent_asin = " . $this->dbquote($product->parent_asin) . "
			// 	order by attr1_name, attr2_name, attr3_name, attr4_name, attr5_name, attr6_name, attr7_name, attr8_name, attr9_name, attr10_name";
			// $res = $this->dbquery($sql);
			// while ($row = mysqli_fetch_object($res)) {
			// 	$row->url = $this->instDir . "/itm/" . $row->asin . "/" . $this->tidyup($row->title) . ".html";
			// 	$product->variations[] = $row;
			// }
		}
		return $product;
	}

	function getCurrentVariation($asin,$parent_asin) {
		$parent_asin = trim($parent_asin);
		$asin = trim($asin);
		$sql = "select * from `" . $this->tablePrefix . "variations` where asin = " . $this->dbquote($asin) . " and parent_asin = " . $this->dbquote($parent_asin);
		$res = $this->dbquery($sql);
		if (!mysqli_num_rows($res)) return false;
		$row = mysqli_fetch_object($res);
		$row->url = $this->instDir . "/itm/" . $row->asin . "/" . $this->tidyup($row->title) . ".html";
		for ($i=1;$i<11;$i++) {
			if ($row->{"attr" . $i . "_name"} && $row->{"attr" . $i . "_value"}) {
				$row->attr[$row->{"attr" . $i . "_name"}] = $row->{"attr" . $i . "_value"};
			}
		}
		return $row;
	}

	// variations to display images with attributes for selection
	function getVariationsImagesAndAttributes($parent_asin,$asin) {
		$parent_asin = trim($parent_asin);
		if (!$parent_asin) return false;
		$variations = $orderby = array();
		$currentVariation = new stdClass();
		for ($i=1;$i<11;$i++) {
			$orderby[] = "attr" . $i . "_name, attr" . $i . "_value"; 
		}
		$sql = "select * from `" . $this->tablePrefix . "variations` where 
			parent_asin = " . $this->dbquote($parent_asin) . "
			order by " . implode(", ", $orderby);
		$res = $this->dbquery($sql);
		while ($row = mysqli_fetch_object($res)) {
			$row->url = $this->instDir . "/itm/" . $row->asin . "/" . $this->tidyup($row->title) . ".html";
			$row->attr = array();
			for ($i=1;$i<11;$i++) {
				if ($row->{"attr" . $i . "_name"} && $row->{"attr" . $i . "_value"}) {
					$row->attr[$row->{"attr" . $i . "_name"}] = $row->{"attr" . $i . "_value"};
				}
			}
			if ($asin && $row->asin == $asin) {
				$currentVariation = $row;
			}
			$variations[] = $row;
		}
		return array($variations,$currentVariation);
	}


	// variation names and attributes for select fields
	function getVariationsAttributesAndValues($parent_asin,$asin) {
		$parent_asin = trim($parent_asin);
		$asin = trim($asin);
		if (!$parent_asin) return false;
		$variations = array();
		$sql = "select * from (
       select parent_asin, attr1_name attr_name, attr1_value attr_value from `" . $this->tablePrefix . "variations`
     UNION
       select parent_asin, attr2_name attr_name, attr2_value attr_value from `" . $this->tablePrefix . "variations`
     UNION
       select parent_asin, attr3_name attr_name, attr3_value attr_value from `" . $this->tablePrefix . "variations`
     UNION
       select parent_asin, attr4_name attr_name, attr4_value attr_value from `" . $this->tablePrefix . "variations`
     UNION
        select parent_asin, attr5_name attr_name, attr5_value attr_value from `" . $this->tablePrefix . "variations`
     UNION
       select parent_asin, attr6_name attr_name, attr6_value attr_value from `" . $this->tablePrefix . "variations`
     UNION
       select parent_asin, attr7_name attr_name, attr7_value attr_value from `" . $this->tablePrefix . "variations`
     UNION
       select parent_asin, attr8_name attr_name, attr8_value attr_value from `" . $this->tablePrefix . "variations`
     UNION
       select parent_asin, attr9_name attr_name, attr9_value attr_value from `" . $this->tablePrefix . "variations`
     UNION
       select parent_asin, attr10_name attr_name, attr10_value attr_value from `" . $this->tablePrefix . "variations`
     ) as t
		where parent_asin = " . $this->dbquote($parent_asin) . " 
			and attr_name is not null 
		order by attr_name, attr_value";
		$res = $this->dbquery($sql);
		while ($row = mysqli_fetch_object($res)) {
			// $row->url = $this->instDir . "/itm/" . $row->asin . "/" . $this->tidyup($row->title) . ".html";
			$variations[$row->attr_name][] = $row->attr_value;
		}

		$selections = $tmpSelections = $currentVariation = array();
		//selections
		$sql = "select * from `" . $this->tablePrefix . "variations` where parent_asin = " . $this->dbquote($parent_asin);
		$res = $this->dbquery($sql);
		while ($row = mysqli_fetch_object($res)) {
			$selectionKeys = $tmpRow = array();
			for ($i=0;$i<11;$i++) { 
				if ($row->{"attr" . $i . "_value"}) {
					$tmpRow[] = $row->{"attr" . $i . "_name"} . ":" . $row->{"attr" . $i . "_value"};
					$selectionKeys[] = $row->{"attr" . $i . "_name"} . ":" . $row->{"attr" . $i . "_value"};
					if ($asin && $row->asin == $asin) {
						$currentVariation[] = $row->{"attr" . $i . "_name"} . ":" . $row->{"attr" . $i . "_value"};
					}
				}
			}
			foreach ($selectionKeys as $selectionElement) {
				$tmpSelections[$selectionElement][] = $selectionKeys;
			}
		}
		//now arrange the selections
		foreach ($tmpSelections as $key => $values) {
			$selections[$key] = array_unique(call_user_func_array('array_merge', $values));
		}

		// echo "<pre>" . print_r($selections,true) . "</pre><hr />";


		return array($variations,$currentVariation,$selections);
	}


	// item url for selected variations
	function getVariationsUrl($parent_asin,$variations,$attrName) {
		$parent_asin = trim($parent_asin);
		if (!$parent_asin) return false;
		if (!$variations) return false;

		// $attrNameFields = $attrValueFields = array();
		// for ($i=1;$i<11;$i++) {
		// 	$attrNameFields[] = "attr"
		// } 
		$add2sql[] = $add2currentSql[] = "parent_asin = " . $this->dbquote($parent_asin);
		foreach ($variations as $variation) {
			if ($variation["name"] && $variation["val"]) {
				$add2sql[] = $this->dbquote($variation["name"]) . " in (attr" . implode("_name, attr", range(1,10)) . "_name)";
				$add2sql[] = $this->dbquote($variation["val"]) . " in (attr" . implode("_value, attr", range(1,10)) . "_value)";
				if ($attrName && $attrName == $variation["name"]) {
					$add2currentSql[] = $this->dbquote($variation["name"]) . " in (attr" . implode("_name, attr", range(1,10)) . "_name)";
					$add2currentSql[] = $this->dbquote($variation["val"]) . " in (attr" . implode("_value, attr", range(1,10)) . "_value)";
				}
			}
		}
		$sql = "select * from `" . $this->tablePrefix . "variations` where " . implode(" and ",$add2sql);
		$res = $this->dbquery($sql);
		if (!mysqli_num_rows($res)) {
			$sql = "select * from `" . $this->tablePrefix . "variations` where " . implode(" and ",$add2currentSql);
			$res = $this->dbquery($sql);
		}
		$row = mysqli_fetch_object($res);
		$item_url = $this->instDir . "/itm/" . $row->asin . "/" . $this->tidyup($row->title) . ".html";
		$success = true;


		// $success = $parent_asin . "<pre>" . print_r($variations,true) . "</pre><hr>$item_url<br />$sql";

 		$this->json = json_encode(array("success" => $success, "item_url" => $item_url));
	} 




	function getProductReviews($asin) {
		$reviews = array();
		$sql = "select * from `" . $this->tablePrefix . "editorial_reviews` where asin = " . $this->dbquote($asin) . " order by pos";
		$res = $this->dbquery($sql);
		while ($row = mysqli_fetch_object($res)) {
			$reviews[] = $row;
		}
		return $reviews;
	}

	function getProductCategories($id) {
		$id = intval($id);
		if (!$id) return false;
		$cat_ids = array();
		$sql = "select category_id from `" . $this->tablePrefix . "category_products` where product_id = '$id'";
		$res = $this->dbquery($sql);
		while ($row = mysqli_fetch_object($res)) {
			$cat_ids[] = $row->category_id;
		}
		return $cat_ids;
	}



	//update/create editorial reviews/product details
	function saveProductReview($data) {
		$reviewId = intval($data["reviewId"]);
		$asin = trim($data["asin"]);
		$source = trim($data["source"]);
		$review = trim($data["review"]);

		$error = array();
		$success = $add2sql = null;
		$newReviewId = 0;

		if (!$reviewId && !$asin) $error[] = "Unsufficient data provided (missing ID and/or ASIN).";
		if (!$source || !$review) $error[] = "Please type in Source/Headline and Review/Product Details to complete operation.<pre>" . print_r($data,true) . "</pre>";
	
		if (!$error) {
			$add2sql = "source = " . $this->dbquote($source) . ", review = " . $this->dbquote($review);
			if ($reviewId) {
				$sql = "update `" . $this->tablePrefix . "editorial_reviews` set $add2sql, edited = now() where id = '$reviewId'";
				$this->dbquery($sql);
				$success = "The editorial product review has been updated successfully.";
			} else {
				$sql = "insert into `" . $this->tablePrefix . "editorial_reviews` set $add2sql, asin = " . $this->dbquote($asin) . ", created = now(), pos = -1";
				$this->dbquery($sql);
				$success = "The editorial product review has been created successfully.";
				$newReviewId = mysqli_insert_id($this->db);
				$this->reposEditorialReviews($asin);
			}
		}

 		$this->json = json_encode(array("success" => $success, "error" => $error ? implode("<br />",$error) : false, "newReviewId" => $newReviewId
			));

	}

	//update custom title (in product list view)
	function updateCustomTitle($data) {
		$itemId = intval($data["itemId"]);
		$customTitle = trim($data["customTitle"]);
		$error = $success = null;

		if (!$itemId) {
			$error = "Insufficient data transmitted <i>itemId not set</i>.";
		} else {
			$sql= "select asin from `" . $this->tablePrefix . "products` where id = '$itemId'";
			$res = $this->dbquery($sql);
			if (!mysqli_num_rows($res)) {
				$error = "The item you are trying to modify cannot be found.";
			} else {
				$row = mysqli_num_rows($res);
				$asin = $row->asin;
			}
		}
		if (!$error) {
			$sql = "update `" . $this->tablePrefix . "products` set
				title_custom = " . $this->dbquote($customTitle) . ",
				edited = now()
				where id = '$itemId'";
			$this->dbquery($sql);
			$success = "The custom title for item <code>$asin</code> has been updated successfully.";
		}
 		$this->json = json_encode(array(
 			"success" => $success, 
 			"error" => $error, 
 			"customTitle" => $customTitle
		));
	}



	//remove editorial review
	function removeEditorialReview($reviewId,$asin) {
		$reviewId = intval($reviewId);
		$asin = trim($asin);
		$success = $error = null;
		if (!$reviewId || !$asin) {
			$error = "Unsufficient data provided (missing ID and ASIN).";
		}
		if (!$error) {
			$sql = "delete from `" . $this->tablePrefix . "editorial_reviews` where id = '$reviewId'";
			$this->dbquery($sql);
			$success = "The editorial review has been deleted successfully.";
			$this->reposEditorialReviews($asin);
		}
 		$this->json = json_encode(array("success" => $success, "error" => $error
 				));

	}

	//order single editorial review a level up or down
	function positionEditorialReview($reviewId,$dir) {
		$reviewId = intval($reviewId);
		$success = $error = null;
		if (!$reviewId) return false;
		// we need the details
		$sql = "select pos, asin from `" . $this->tablePrefix . "editorial_reviews` where id = '$reviewId'";
		$res = $this->dbquery($sql);
		$reviewData = mysqli_fetch_object($res);
		if (!$reviewData) return false;
		if ($dir == "up") {
			$sql = "update `" . $this->tablePrefix . "editorial_reviews` set pos = pos - 1 where id = '$reviewId'";
			$this->dbquery($sql);
			//repos lower positioned one
			$sql = "update `" . $this->tablePrefix . "editorial_reviews` set pos = pos + 1 where pos = '" . ($reviewData->pos - 1) . "' and id != '$reviewId' and asin = " . $this->dbquote($reviewData->asin);
			$this->dbquery($sql);
		} elseif ($dir == "down") {
			$sql = "update `" . $this->tablePrefix . "editorial_reviews` set pos = pos + 1 where id = '$reviewId'";
			$this->dbquery($sql);
			//repos higher positioned one
			$sql = "update `" . $this->tablePrefix . "editorial_reviews` set pos = pos - 1 where pos = '" . ($reviewData->pos + 1) . "' and id != '$reviewId' and asin = " . $this->dbquote($reviewData->asin);
			$this->dbquery($sql);
		}
		//for the case of a mistake
		$this->reposEditorialReviews($reviewData->asin);

 		$this->json = json_encode(array("success" => true));

	}

	//resort reviews
	//to reorder after deletion, repositioning or adding of editorial reviews
	function reposEditorialReviews($asin) {
		if (!$asin) return false;
		$pos = 1;
		$sql = "select * from `" . $this->tablePrefix . "editorial_reviews` where asin = " . $this->dbquote($asin) . " order by pos";
		$res = $this->dbquery($sql);
		while ($row = mysqli_fetch_object($res)) {
			$sql = "update `" . $this->tablePrefix . "editorial_reviews` set pos = '" . $pos++ . "' where id = '" . $row->id . "'";
			$this->dbquery($sql);
		}
	}





	function getImgSizeUrl($url,$size=false) {
		$imgUrls = array();
		// $sizes name ... max height,width
		$sizes = array("xxs" => 40,
									 "xs" => 80,
									 "sm200" => 200,
									 "sm" => 240,
									 "md" => 420,
									 "lg" => 640);
		foreach ($sizes as $key => $val) {
			$imgUrls[$key] = preg_replace('/(\..{3})$/',"._SL" . $val . "_$1",$url);
		}
		if ($size) {
			if (isset($imgUrls[$key])) return $imgUrls[$size];
			else return $url;
		} else {
			return $imgUrls;
		}
	}


	function drawPagination($pages,$p) {
	  if ($pages < 2) return false;   //nothing to page
	  if ($pages > 8) {
	    //interpage calc
	    $pstart = $p - 2;
	    if ($pstart <= 1) $pstart = 2;
	    $pend = $p + 2;
	    if ($pend > $pages) $pend = $pages;
	    $pages_after_last_interpage = ($pend+1 >= $pages ? true : false);
	    $pages_before_first_interpage = ($pstart-2 >= 1 ? true : false);
	  
	    $NAV_PAGES[] = array("P" => 1,
	                         "ACTIVE" => $p == 1);
	    if ($pages_before_first_interpage) $NAV_PAGES[] = array("SEP" => true, "P" => false, "ACTIVE" => false);
	                         
	    for ($i=$pstart;$i<=$pend;$i++) {
	      $NAV_PAGES[] = array("P" => $i,
	                           "ACTIVE" => $p == $i);
	      if ($i == $pend && !$pages_after_last_interpage) $NAV_PAGES[] = array("SEP" => true, "P" => false, "ACTIVE" => false);
	    }
	    if ($pend < $pages) {
	      $NAV_PAGES[] = array("P" => $pages,
	      										 "ACTIVE" => false);
	    }
	  } else {
	    //into template array
	    for ($i=1;$i<=$pages;$i++) {
	      $NAV_PAGES[] = array("P" => $i,
	                           "ACTIVE" => $p == $i);
	    }
	  }
	  return $NAV_PAGES;
	}


	function getProductsMapData() {
		$add2sql = $productsMapData = array();

		list($mainCategories,$subCategories,$categories) = $this->getCategories();
		if ($categories) {

			function myCatSorter($a,$b) {
				return $a->internal_parent_child_sort_coding*10000 - $b->internal_parent_child_sort_coding*10000;
			}
			uasort($categories,"myCatSorter");

			foreach ($categories as $cat) {
				if ($cat->parent) {
					$productsMapData[$cat->id]["catDescr"] = $cat->descr;
					$productsMapData[$cat->id]["catIntro"] = "in <em>" . $categories[$cat->parent]->descr . "</em>" . ($cat->short_intro ? " | " . $cat->short_intro : "");
				} else {
					$productsMapData[$cat->id]["catDescr"] = $cat->descr;
					$productsMapData[$cat->id]["catIntro"] = $cat->short_intro;
				}
				$productsMapData[$cat->id]["catUrl"] = $this->instDir . "/cat" . $cat->id . "/" . $this->tidyup($cat->descr) . ".html";
				$sql = "select id, asin, title, title_custom, image_url from `" . $this->tablePrefix . "products` where
					id in (select product_id from `" . $this->tablePrefix . "category_products` where category_id = '" . $cat->id . "') and
					imported is not null
					order by if (title_custom is not null and title_custom != '', lower(title_custom), lower(title))";
				$res = $this->dbquery($sql);
				if (mysqli_num_rows($res)) {
					while ($item = mysqli_fetch_object($res)) {
		    		$item->titleDB = $item->title_custom ?: $item->title;
						$item->url = $this->instDir . "/itm/" . $item->asin . "/" . $this->tidyup($item->titleDB) . ".html";
						$productsMapData[$cat->id]["products"][] = $item;
					}
				} else {
					unset($productsMapData[$cat->id]);
				}
			}
		}
		//uncategorized items - use product_groups to order them
		$sql = "select id, asin, title, title_custom, product_group from `" . $this->tablePrefix . "products` where
			id not in (select product_id from `" . $this->tablePrefix . "category_products`) and
			imported is not null
			order by product_group,
			if (title_custom is not null and title_custom != '',lower(title_custom),lower(title))";
		$res = $this->dbquery($sql);
		if (mysqli_num_rows($res)) {
			while ($item = mysqli_fetch_object($res)) {
				if (!isset($productsMapData[$item->product_group])) {
					$productsMapData[$item->product_group]["catDescr"] = $this->lang("hl-prodmap-unsorted-itemsblock");
					$productsMapData[$item->product_group]["catIntro"] = $item->product_group;
				}
	    	$item->titleDB = $item->title_custom ?: $item->title;
				$item->url = $this->instDir . "/itm/" . $item->asin . "/" . $this->tidyup($item->titleDB) . ".html";
				$productsMapData[$item->product_group]["products"][] = $item;
			}
			
		}

		return $productsMapData;
	}




/******* cart stuff [+] *************/

	//get cart Ids and Keys - initiate creation of cart if needed
	function getCartKeys() {
		$CartId = $HMAC = null;
		$cartKeys = array();
    if ($_COOKIE["amzCartKeys"] && preg_match('/\|\|\|/', $_COOKIE["amzCartKeys"])) {
    	list($cartKeys["CartId"],$cartKeys["HMAC"]) = explode("|||",$_COOKIE["amzCartKeys"]);
    }
    return $cartKeys;
	}


	//create cart and add item
	function createAndAddItem2cart($asin) {
		$error = false;

		$XMLurl = $this->amzGetXMLUrl("createCart",array("ASIN" => $asin, "quantity" => 1));


		// //CURL Version
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $XMLurl);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    // get the result of http query
    $output = curl_exec($ch);
    curl_close($ch);
    
    if (!preg_match('/^</',$output)) {
      $this->error = "Data of $url not completed loaded.<br />
        Output:<pre>$output</pre>";
    } else {
      $xmlData = simplexml_load_string($output);
    }

    if (!preg_match('/^</',$output)) {
      $this->error = "Data of $url not completed loaded.<br />
        Output:<pre>$output</pre>";
    } else {
      $xmlData = simplexml_load_string($output);
    }

    if ($xmlData->Cart->Request->Errors) {
    	// $this->cartError = $xmlData->Cart->Request->Errors->Error->Message;
    	foreach ($xmlData->Cart->Request->Errors as $errItem) {
    		$this->cartError .= strval($errItem->Error->Message) . "<br />";
    	}
    } else {
	    $CartId = strval($xmlData->Cart->CartId);
	    $HMAC = strval($xmlData->Cart->HMAC);
      setcookie("amzCartKeys", $CartId . "|||" . $HMAC, time()+60*60*24*7, $this->instDir);  //expires in approx. one week :)
    }
    return $cartData;
	}


	//get cart (content)
	function getCart() {
		$cartData = array();
		$cartKeys = $this->getCartKeys();
		if ($cartKeys) {
			$XMLurl = $this->amzGetXMLUrl("getCart",$cartKeys);

			// //CURL Version
	    $ch = curl_init();
	    curl_setopt($ch, CURLOPT_URL, $XMLurl);
	    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	    // get the result of http query
	    $output = curl_exec($ch);
	    curl_close($ch);
	    
	    if (!preg_match('/^</',$output)) {
	      $this->error = "Data of $url not completed loaded.<br />
	        Output:<pre>$output</pre>";
	    } else {
	      $xmlData = simplexml_load_string($output);
	    }

	    if (!preg_match('/^</',$output)) {
	      $this->error = "Data of $url not completed loaded.<br />
	        Output:<pre>$output</pre>";
	    } else {
	      $xmlData = simplexml_load_string($output);
	    }

	    // if ($xmlData->Cart->Request->Errors) {
	    // 	// $this->cartError = $xmlData->Cart->Request->Errors->Error->Message;
	    // 	foreach ($xmlData->Cart->Request->Errors as $errItem) {
	    // 		$this->cartError .= strval($errItem->Error->Message) . "<br />";
	    // 	}
	    // } else {
		   //  $CartId = strval($xmlData->Cart->CartId);
		   //  $HMAC = strval($xmlData->Cart->HMAC);
	    //   setcookie("amzCartKeys", $CartId . "|||" . $HMAC, time()+60*60*24*60, $zstore->instDir);  //expires in approx. two months :)
	    // }

	    $cartData["PurchaseURL"] = strval($xmlData->Cart->PurchaseURL);
	    $cartData["Items"] = array();
	    if(isset($xmlData->Cart->CartItems->CartItem)) {
		    foreach ($xmlData->Cart->CartItems->CartItem as $item) {
		    	$item = (object) (array) $item;
		    	//get additional information to single items
		    	$itemData = $this->getProductData($item->ASIN,0,"small");
		    	$item->image_url = $itemData->image_url;
		    	$item->titleDB = $itemData->title_custom ?: $itemData->title;
					$item->url = $this->instDir . "/itm/" . $item->ASIN . "/" . $this->tidyup($item->titleDB) . ".html";

		    	$cartData["Items"][] = $item;
		    }
		  }

	    // $xmlData->Cart->CartItems->CartItem;
	    $cartData["SubTotal"] = $xmlData->Cart->CartItems->SubTotal;
	    $cartData["itemsCount"] = count($cartData["Items"]);

	    //something saved for later??? - will be implemented in a future version
	    $cartData["savedItems"] = array();
	    if (isset($xmlData->Cart->SavedForLaterItems)) {
				foreach ($xmlData->Cart->SavedForLaterItems->SavedForLaterItem as $item) {
		    	$item = (object) (array) $item;
		    	//get additional information to single items
		    	$itemData = $this->getProductData($item->ASIN,0,"small");
		    	$item->image_url = $itemData->image_url;
	    		$item->titleDB = $itemData->title_custom ?: $itemData->title;
					$item->url = $this->instDir . "/itm/" . $item->ASIN . "/" . $this->tidyup($item->titleDB) . ".html";

		    	$cartData["savedItems"][] = $item;
		    }
	    }



		} else {
	    $cartData["itemsCount"] = 0;
	    return $cartData;
		}
		// $cartData["xmlData"] = $xmlData;
	  return $cartData;
	}

	
	//add something to cart
	function add2cart($asin) {
		$error = $success = null;
		$asin = trim($asin);

		if (!$asin) {
			$error = $this->lang("error_unsufficient_data");
		}
		if (!$error) {
			//is there a cart?
			$cartKeys = $this->getCartKeys();
			if ($cartKeys) {
				// add item to existing cart 

				$XMLurl = $this->amzGetXMLUrl("add2cart",array("ASIN" => $asin, "quantity" => 1, "CartId" => $cartKeys["CartId"], "HMAC" => $cartKeys["HMAC"]));

				// //CURL Version
		    $ch = curl_init();
		    curl_setopt($ch, CURLOPT_URL, $XMLurl);
		    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		    // get the result of http query
		    $output = curl_exec($ch);
		    curl_close($ch);
		    
		    if (!preg_match('/^</',$output)) {
		      $this->error = "Data of $url not completed loaded.<br />
		        Output:<pre>$output</pre>";
		    } else {
		      $xmlData = simplexml_load_string($output);
		    }

		    if (!preg_match('/^</',$output)) {
		      $this->error = "Data of $url not completed loaded.<br />
		        Output:<pre>$output</pre>";
		    } else {
		      $xmlData = simplexml_load_string($output);
		    }

		    if ($xmlData->Cart->Request->Errors) {
		    	// $this->cartError = $xmlData->Cart->Request->Errors->Error->Message;
		    	foreach ($xmlData->Cart->Request->Errors as $errItem) {
		    		//NOT already in cart????
		    		if (strval($errItem->Error->Code) != "AWS.ECommerceService.ItemAlreadyInCart") {
		    			$error .= strval($errItem->Error->Code . "<br />" . $errItem->Error->Message) . "<br />";
		    		}
		    	}
		    } 
		    if (!$error) {
					$success = sprintf($this->lang("success_item_x_added2cart"),$asin);
		    }

			} else {
				//create cart and add item to cart
				$this->createAndAddItem2cart($asin);
				if ($this->cartError) {
					$error = $this->cartError;
				} else {
					$success = sprintf($this->lang("success_item_x_added2cart"),$asin);
				}
			}
		}

 		$this->json = json_encode(array("success" => $success, "error" => $error,
		));

	}


	//remove something from cart
	function removeFromCart($cartItemId) {
		$error = $success = $subTotal = null;
		$cartItemId = trim($cartItemId);
		$itemsCount = 0;

		if (!$cartItemId) {
			$error = $this->lang("error_unsufficient_data");
		}
		if (!$error) {
			//is there a cart?
			$cartKeys = $this->getCartKeys();
		}
		if (!$cartKeys) {
			//cart not found - no need to delete something - but inform the customer
			$error = $this->lang("error_remove_item_no_cart");
		}
		if (!$error) {
			$XMLurl = $this->amzGetXMLUrl("removeFromCart",array("cartItemId" => $cartItemId, "CartId" => $cartKeys["CartId"], "HMAC" => $cartKeys["HMAC"]));

			// //CURL Version
	    $ch = curl_init();
	    curl_setopt($ch, CURLOPT_URL, $XMLurl);
	    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	    // get the result of http query
	    $output = curl_exec($ch);
	    curl_close($ch);
	    
	    if (!preg_match('/^</',$output)) {
	      $this->error = "Data of $url not completed loaded.<br />
	        Output:<pre>$output</pre>";
	    } else {
	      $xmlData = simplexml_load_string($output);
	    }

	    if (!preg_match('/^</',$output)) {
	      $this->error = "Data of $url not completed loaded.<br />
	        Output:<pre>$output</pre>";
	    } else {
	      $xmlData = simplexml_load_string($output);
	    }

	    if ($xmlData->Cart->Request->Errors) {
	    	foreach ($xmlData->Cart->Request->Errors as $errItem) {
	    		$error .= strval($errItem->Error->Code . "<br />" . $errItem->Error->Message) . "<br />";
	    	}
	    } 
	    if (!$error) {

				$items = array();
		    foreach ($xmlData->Cart->CartItems->CartItem as $item) {
		    	$items[] = $item;
		    }

		    $subTotal = strval($xmlData->Cart->CartItems->SubTotal->FormattedPrice);
		    $itemsCount = count($items);

				$success = $this->lang("success_item_removed_from_cart");

				if (!$itemsCount) {
					$success .= "<br />" . $this->lang("success_info_cart_empty_now");
				}

	    }

		}

 		$this->json = json_encode(array("success" => $success, "error" => $error, "subTotal" => $subTotal, "itemsCount" => $itemsCount 
		));

	}



	//empty cart
	function emptyCart() {
		$error = $success = null;

		$cartKeys = $this->getCartKeys();

		if (!$cartKeys) {
			//cart not found - no need to delete something - but inform the customer
			$error = $this->lang("error_remove_item_no_cart");
		}
		if (!$error) {
			$XMLurl = $this->amzGetXMLUrl("emptyCart",array("CartId" => $cartKeys["CartId"], "HMAC" => $cartKeys["HMAC"]));

			// //CURL Version
	    $ch = curl_init();
	    curl_setopt($ch, CURLOPT_URL, $XMLurl);
	    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	    // get the result of http query
	    $output = curl_exec($ch);
	    curl_close($ch);
	    
	    if (!preg_match('/^</',$output)) {
	      $this->error = "Data of $url not completed loaded.<br />
	        Output:<pre>$output</pre>";
	    } else {
	      $xmlData = simplexml_load_string($output);
	    }

	    if (!preg_match('/^</',$output)) {
	      $this->error = "Data of $url not completed loaded.<br />
	        Output:<pre>$output</pre>";
	    } else {
	      $xmlData = simplexml_load_string($output);
	    }

	    if ($xmlData->Cart->Request->Errors) {
	    	foreach ($xmlData->Cart->Request->Errors as $errItem) {
	    		$error .= strval($errItem->Error->Code . "<br />" . $errItem->Error->Message) . "<br />";
	    	}
	    } 
	    if (!$error) {
				$success = $this->lang("success_cart_emptied");
	    }

		}

 		$this->json = json_encode(array("success" => $success, "error" => $error
		));
	}


	//modify number of items in cart
	function modifyCartItem($cartItemId,$quantity) {
		$error = $success = $subTotal = $itemSubTotal = null;
		$cartItemId = trim($cartItemId);
		$quantity = intval($quantity);
		$itemsCount = 0;

		if (!$cartItemId) {
			$error = $this->lang("error_unsufficient_data");
		}
		if (!$error) {
			//is there a cart?
			$cartKeys = $this->getCartKeys();
		}
		if (!$cartKeys) {
			//cart not found - no possibility to update something - but inform the customer
			$error = $this->lang("error_remove_item_no_cart");
		}
		if (!$error) {
			$XMLurl = $this->amzGetXMLUrl("modifyCartItem",array("cartItemId" => $cartItemId, "quantity" => $quantity, "CartId" => $cartKeys["CartId"], "HMAC" => $cartKeys["HMAC"]));

			// //CURL Version
	    $ch = curl_init();
	    curl_setopt($ch, CURLOPT_URL, $XMLurl);
	    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	    // get the result of http query
	    $output = curl_exec($ch);
	    curl_close($ch);
	    
	    if (!preg_match('/^</',$output)) {
	      $this->error = "Data of $url not completed loaded.<br />
	        Output:<pre>$output</pre>";
	    } else {
	      $xmlData = simplexml_load_string($output);
	    }

	    if (!preg_match('/^</',$output)) {
	      $this->error = "Data of $url not completed loaded.<br />
	        Output:<pre>$output</pre>";
	    } else {
	      $xmlData = simplexml_load_string($output);
	    }

	    if ($xmlData->Cart->Request->Errors) {
	    	foreach ($xmlData->Cart->Request->Errors as $errItem) {
	    		$error .= strval($errItem->Error->Code . "<br />" . $errItem->Error->Message) . "<br />";
	    	}
	    } 
	    if (!$error) {

				$items = array();
		    foreach ($xmlData->Cart->CartItems->CartItem as $item) {
		    	$items[] = $item;
		    	//get current element
	    		if ($item->CartItemId == $cartItemId) {
						$itemSubTotal = strval($item->ItemTotal->FormattedPrice);
	    		}
		    	
		    }

		    $subTotal = strval($xmlData->Cart->CartItems->SubTotal->FormattedPrice);
		    $itemsCount = count($items);

				if ($quantity > 0) {
					$success = $this->lang("success_item_number_changed");
				} else {
					$success = $this->lang("success_item_removed_from_cart");
					if (!$itemsCount) {
						$success .= "<br />" . $this->lang("success_info_cart_empty_now");
					}
				}

	    }

		}

 		$this->json = json_encode(array("success" => $success, "error" => $error, "subTotal" => $subTotal, "itemsCount" => $itemsCount , "itemSubTotal" => $itemSubTotal
		));

	}




/******* cart stuff [-] *************/





/******* favorites [+] *************/

	//add an item to favorites
	function add2favs($asin) {
		$error = $success = null;
		$asin = trim($asin);
		if (!$asin) {
			$error = $this->lang("error_unsufficient_data");
		}
		if (!$error) {
    	setcookie("amzFavItems[$asin]", $asin, time()+60*60*24*31, $this->instDir);  //expires in approx. one month :)
			$success = sprintf($this->lang("success_item_x_added2favorites"),$asin);
		}

 		$this->json = json_encode(array("success" => $success, "error" => $error, "newTitle" => $this->lang("title_remove_from_favorites")
		));

	}

	//get just the asins from the favorites
	function removeFromFavs($asin) {
		$error = $success = null;
		$asin = trim($asin);
		$noFavorites = false;
		if (!$asin) {
			$error = $this->lang("error_unsufficient_data");
		}
		if (!$error) {
			setcookie("amzFavItems[$asin]", null, -1, $this->instDir); 
			unset($_COOKIE["amzFavItems"][$asin]);
			$success = sprintf($this->lang("success_item_x_removed_from_favorites"),$asin);
			if (!$this->getAsinsFromFavs()) {
				$noFavorites = true;
			}
		}

 		$this->json = json_encode(array("success" => $success, "error" => $error, 
 			"newTitle" => $this->lang("title_add2favorites"),
 			"noFavorites" => $noFavorites
		));


	}

	//get just the asins from the favorites
	function getAsinsFromFavs() {
		$asins = array();
		if (array_key_exists("amzFavItems", $_COOKIE) && $_COOKIE["amzFavItems"]) {
			foreach ($_COOKIE["amzFavItems"] as $asin) {
				$asins[] = $asin;
			}
		}
		return $asins;
	}


/******* favorites [-] *************/




	/*** mainly admin stuff [+] ***/


	//just for the admin config
	function getMarketplaces() {
		$marketplaces = array();
		$sql = "select * from `" . $this->tablePrefix . "amazon_marketplaces` order by descr";
		$res = $this->dbquery($sql);
		while ($row = mysqli_fetch_object($res)) {
			$marketplaces[] = $row;
		}
		return $marketplaces;
	}

	function getAmazonSearchIndexes() {
		$searchIndexes = array();
		$sql = "select * from `" . $this->tablePrefix . "amazon_searchindexes` order by descr";
		$res = $this->dbquery($sql);
		while ($row = mysqli_fetch_object($res)) {
			$searchIndexes[] = $row;
		}
		return $searchIndexes;
	}

	//config data 
	function getConfigData() {
		if (!$this->auth) return false;
		$sql = "select * from `" . $this->tablePrefix . "config` where id = '" . intval($this->configId) . "'";
		$res = $this->dbquery($sql);
		$config = mysqli_fetch_object($res);
		return $config;
	}


	//config data for the web project
	function setConfig() {
		$sql = "select * from `" . $this->tablePrefix . "config` where id = '" . intval($this->configId) . "'";
		$res = $this->dbquery($sql,$this->setup);
		if ($res) {
			$config = mysqli_fetch_object($res);
			$this->config = $config;
			$this->config->layout = $this->layout;	//to remove if it set in db
			$this->lang = $config->lang;
		}
		if ($this->setup && !$config->version) {
			$this->version = number_format($this->initVersion,2);
		} else {
			$this->version = number_format($config->version,2);
		}
	}

	//site settings
	//params ... assoc array, keys: title, description
	function setSite($params = false) {
		$site = new stdClass();
		if (!$params) {
			$site->title = $this->config->site_title;
			$site->meta_description = $this->config->meta_description;
			$site->meta_keywords = $this->config->meta_keywords;
		} elseif (is_array($params)) {

			$site->title = isset($params["title"]) ? $params["title"] : null;
			if ($site->title) $site->title .= " | ";
			$site->title .= $this->config->site_title;

			$site->meta_description = isset($params["description"]) && $params["description"] ? $params["description"] : $this->config->meta_description;

			$site->meta_keywords = isset($params["keywords"]) && $params["keywords"] ? $params["keywords"] : $this->config->meta_keywords;

		}
		$site->author = $this->config->site_author;

		$this->site = $site;
	}

	function getLanguages($return = false) {
		$languages = array();
		$sql = "select lang_key from `" . $this->tablePrefix . "lang` group by lang_key order by lang_key";
		$res = $this->dbquery($sql);
		while ($row = mysqli_fetch_object($res)) {
			$languages[] = $row->lang_key;
		}
		if ($return == "json") {
	 		$this->json = json_encode(array("languages" => $languages));
		} else {
			return $languages;
		}
	}

	//draw a.k.a. page
	function getLanguageEntries($lang_key,$orderBy = "short_key", $search = null, $offset, $perPage) {
		$lang_key = trim(strtolower($lang_key));
		if (!$lang_key) return false;
		$langItems = array();

		$add2sql = $limitSql = null;

		
		//totals
		$sql = "select * from `" . $this->tablePrefix . "lang` where lang_key = " . $this->dbquote($lang_key);
		$res = $this->dbquery($sql);
		$recordsTotal = $recordsFiltered = mysqli_num_rows($res);


		if ($search) {
			$add2sql = " and (lower(short_key) like " . $this->dbquote("%$search%") . " or lower(descr) like " . $this->dbquote("%$search%") . ")";

			$sql = "select * from `" . $this->tablePrefix . "lang` where lang_key = " . $this->dbquote($lang_key) . " $add2sql";
			$res = $this->dbquery($sql);
			$recordsFiltered = mysqli_num_rows($res);
		}
		
		if ($perPage) {
      $limitSql = " limit $offset, $perPage";
		}


		$sql = "select * from `" . $this->tablePrefix . "lang` where lang_key = " . $this->dbquote($lang_key) . " $add2sql order by $orderBy $limitSql";
		$res = $this->dbquery($sql);
		while ($row = mysqli_fetch_object($res)) {
			$langItems[$row->id] = $row;
		}
		return array(
			$langItems,
			$recordsTotal,
			$recordsFiltered
			);

	}

	function getLanguageEntriesNum($lang_key) {
		$lang_key = trim(strtolower($lang_key));
		if (!$lang_key) return false;
		
		//totals
		$sql = "select * from `" . $this->tablePrefix . "lang` where lang_key = " . $this->dbquote($lang_key);
		$res = $this->dbquery($sql);
		return mysqli_num_rows($res);
	}



	//get single language key data
	function getSingleLangShortKeyData($langId,$lang_key = "en") {
		$error = $langData = false;
		$langId = intval($langId);
		if(!$langId) $error = true;

		if (!$error) {
			$sql = "select * from `" . $this->tablePrefix . "lang` where id = '$langId'";
			$res = $this->dbquery($sql);
			if ($res && mysqli_num_rows($res)) {
				$langData = mysqli_fetch_object($res);
			}
		}
		$this->json = json_encode(array("error" => $error, "langData" => $langData));
	}



	function getMissingLanguageKeys($lang_key) {
		$lang_key = trim(strtolower($lang_key));
		$langItems = array();
		$sql = "select * from `" . $this->tablePrefix . "lang` where lang_key = 'en' and 
			short_key not in (select short_key from `" . $this->tablePrefix . "lang` where lang_key = " . $this->dbquote($lang_key) . ")";
		$res = $this->dbquery($sql);
		while ($row = mysqli_fetch_object($res)) {
			$langItems[] = $row;
		}
		return $langItems;
	}


		/****
	 * to display different parts identified by the key in different languages
	 * @param
	 * $key .... strval ... the short key of the string to display in the current language
	 * $args ... array .... [optional] using to replace parts in the translated string with sprintf - 
	 *																			please use the following syntax {a:1} (where's the number is the position/number is the element to replace)
	 *																			{a:1} ... %1$s
	 *																			{a:2} ... %2$s
	 *																			{a:.... 
	 * $update . array ....	[optional] assoc -> possible keys: lang [optional] (de,en,...) if not given current language will be used,
	 *																																when set, this language will be used to search for the correct phrase too
	 *																												 str ... string to add/update for given $key and language (current or lan-key-val)
	 * note on the translations - when sprintf arguments needed please use the following syntax
	 *					{a:1} (where's the number is the position/number is the element to replace)
	 *																			{a:1} ... %1$s
	 *																			{a:2} ... %2$s
	 *																			{a:.... 
	 * returns translated string
	 ****/
  function lang($key,$update=null) {
  	$key = trim($key);
  	if (!$key) return false;
  	$translation = false;
  	$lang = $this->lang;
  	if ($update) {
    	if (is_array($update)) {
  			$updateStr = $update["str"];
  			$lang = $update["lang"];
  		} else {
  			$updateStr = $update;
  		}
  	}
  	$sql = "select * from `" . $this->tablePrefix . "lang` where lang_key = " . $this->dbquote($lang) . " and short_key = " . $this->dbquote($key);
    $res = $this->dbquery($sql);
    if (mysqli_num_rows($res)) {
    	$row = mysqli_fetch_object($res);
    	//update item???
    	if (isset($updateStr)) {
	    	$sql = "update `" . $this->tablePrefix . "lang` set descr = " . $this->dbquote($updateStr) . " where lang_key = " . $this->dbquote($lang) . " and short_key = " . $this->dbquote($key);
    		$this->dbquery($sql);
    		$translation = $updateStr;
    	} else {
    		$translation = $row->descr;
    	}
    } else {
  		$sql = "insert into `" . $this->tablePrefix . "lang` set lang_key = " . $this->dbquote($lang) . ", short_key = " . $this->dbquote($key) . ", created = now()";
	    if (isset($updateStr)) {
    		$sql .= ", descr = " . $this->dbquote($updateStr);
    		$translation = $updateStr;
    	}
    	$this->dbquery($sql);
    }
    if ($translation) {

    	//prepare for sprintf
   		$translation = preg_replace('/\{a:(\d+)\}/','%$1\$s',$translation);

    	return $translation;
    } else {
    	return "[LANG-" . $lang . "::" . $key . "]";
    }
  }


  function editLanguageEntry($data) {
  	$error = array();
  	$newLangId = 0;
  	$success = $dataOrderShortKey = $dataOrderLangDescr = null;
		if (!$this->auth) {
			$error[] = "Please (re-)autenticate.";
		}
  	$id = intval($data["langId"]);
  	$short_key = trim($data["langShortKey"]);
  	$descr = trim($data["langDescr"]);
  	$lang_key = trim(strtolower($data["langKey"]));

  	if (!$short_key) $error[] = "Please type in the short_key for the language item.";
  	if (!$id && !$lang_key) {
  		$error[] = "The for a creation needed language key is missing.";
  	}

  	if (!$error) {
  		$add2sql = "short_key = " . $this->dbquote($short_key) . ",
  			descr = " . $this->dbquote($descr);

  		//edit
  		if ($id) {
  			$sql = "update `" . $this->tablePrefix . "lang` set $add2sql where id = '$id'";
  			$this->dbquery($sql);
  			$success = "The language item has been updated successfully.";
  		} else {
  			$sql = "insert into `" . $this->tablePrefix . "lang` set $add2sql, lang_key = " . $this->dbquote($lang_key) . ", created = now()";
  			$this->dbquery($sql);
				$newLangId = mysqli_insert_id($this->db);
  			$success = "The language item has been updated successfully.";
  		}

  		$dataOrderShortKey = strtolower($short_key);
  		$dataOrderLangDescr = strtolower(strip_tags($descr));

  	}

 		$this->json = json_encode(array("success" => $success, "error" => $error ? implode("<br />",$error) : false,
 			"dataOrderShortKey" => $dataOrderShortKey,
 			"dataOrderLangDescr" => $dataOrderLangDescr,
 			"langShortKey" => $short_key,
 			"langDescr" => $descr,
 			"newLangId" => $newLangId

 		));
  }


  function removeLanguageEntry($id) {
  	$id = intval($id);
  	$error = array();
  	$success = null;
		if (!$this->auth) {
			$error[] = "Please (re-)autenticate.";
		}
  	if (!$id) {
			$error[] = "Insufficient Data sent.";
  	}
  	if (!$error) {
  		//get short key
  		$sql = "select * from `" . $this->tablePrefix . "lang` where id = '$id'";
			$res = $this->dbquery($sql);
			if (mysqli_num_rows($res)) {
				$row = mysqli_fetch_object($res);
				$short_key = $row->short_key;
	  		//in all languages
	  		// $sql = "delete from `" . $this->tablePrefix . "lang` where id = '$id' or short_key = " . $this->dbquote($short_key);
	  		//only in current language
	  		$sql = "delete from `" . $this->tablePrefix . "lang` where id = '$id'";
				$this->dbquery($sql);
				$success = "The language item and all of its siblings has been deleted successfully.<br />" . mysqli_affected_rows($this->db) . " affected in total";
			} else {
				$error[] = "The requested language item cannot be found.";
			}
  	}
  	$this->json = json_encode(array("success" => $success, "error" => $error ? implode("<br />",$error) : false,
 		));
  }


  function removeLanguage($langKey) {
  	$error = array();
  	$success = null;
		if (!$this->auth) {
			$error[] = "Please (re-)autenticate.";
		}
  	if (!$langKey) {
			$error[] = "Insufficient Data sent.";
  	} elseif ($langKey == "en") {
			$error[] = "The <strong>en</strong> language cannot be removed. It's the default zStore default/base language.";
  	} else {
  		//existing
  		$sql = "select id from `" . $this->tablePrefix . "lang` where lang_key = " . $this->dbquote($langKey);
  		$res = $this->dbquery($sql);
  		if (!mysqli_num_rows($res)) {
  			$error[] = "The language key <strong>$langKey</strong> cannot be found.";
  		} elseif ($langKey == $this->lang ) {
  			$error[] = "The language you are trying to remove is set as site language.<br />Please change the site language in the config first to proceed.";
  		}
  	}

  	if (!$error) {
  		$sql = "delete from `" . $this->tablePrefix . "lang` where lang_key = " . $this->dbquote($langKey);
			$this->dbquery($sql);
			$success = "The language an all its keys have been removed successfully.";
  	}
  	$this->json = json_encode(array("success" => $success, "error" => $error ? implode("<br />",$error) : false,
 		));
  }


  function addNewLanguage($langKey,$langCopyBaseKey) {
  	$error = array();
  	$langKey = strtolower($langKey);
  	if (strlen($langKey) > 5) $error[] = "Please type in a language key with five digits in total.";
  	elseif (preg_match('/[^a-z0-9]/',$langKey)) {
  		$error[] = "Please use only letters and numbers for the language key.";
  	}
  	if (!$langCopyBaseKey) $error[] = "Please select a base language for the new language.";

  	//check existing langkeys
  	if (!$error) {
  		$languages = $this->getLanguages();
  		if (in_array($langKey,$languages)) {
  			$error[] = "This language key is already in use.";
  		}
  		if (!in_array($langCopyBaseKey, $languages)) {
  			$error[] = "This base language cannot be found.";
  		}
  	}

  	if (!$error) {
  		$sql = "insert into `" . $this->tablePrefix . "lang` (lang_key, short_key, descr, created)
  			select " . $this->dbquote($langKey) . ", short_key, descr, now() 
  			from `" . $this->tablePrefix . "lang`
  			where lang_key = " . $this->dbquote($langCopyBaseKey);
  		$this->dbquery($sql);
  		$success = "The language has been added successfully.";
  	}
  	$this->json = json_encode(array("success" => $success, "error" => $error ? implode("<br />",$error) : false,
 		));
  }










	//check amazon api data agains amazon webservice servers
	function checkApiData($data) {
		$error = array();
		$success = false;
		if (!$this->auth) {
			$error[] = "Please (re-)autenticate.";
		}
		if (!$error) {
			$amz_marketplace = trim($data["amz_marketplace"]);
			$amz_tag = trim($data["amz_tag"]);
			$amz_access_key = trim($data["amz_access_key"]);
			$amz_secret_key = trim($data["amz_secret_key"]);

			if (!$amz_marketplace) $error[] = "Please select your amazon marketplace.";
			if (!$amz_tag) $error[] = "Please type in your amazon associate tag.";
			if (!$amz_access_key) $error[] = "Please type in the access key id.";
			if (!$amz_secret_key) $error[] = "Please type in the secret key.";
		}
		
		if (!$error) {

			$XMLurl = $this->amzGetXMLUrl("apiCheck",
				array("amz_marketplace" => $amz_marketplace,
					"amz_access_key" => $amz_access_key,
					"amz_tag" => $amz_tag,
					"amz_secret_key" => $amz_secret_key)
				);

			//CURL Version
	    $ch = curl_init();
	    curl_setopt($ch, CURLOPT_URL, $XMLurl);
	    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	    // get the result of http query
	    $output = curl_exec($ch);
	    curl_close($ch);
	    
	    if (!preg_match('/^</',$output)) {
	      $error[] = "Data of $url not completed loaded.<br />
	        Output:<pre>$output</pre>";
	    } else {
	     $xmlData->Items->Request->Errors = simplexml_load_string($output);
	    }

	    if ($xmlData->Items->Request->Errors->Error) {
	    	// $error[] = print_r($xmlData->Items->Request->Errors,true);
				$xmlError = true;
		    $response = $xmlData->Items->Request->Errors->Error;
	    } else {
	    	$response = $xmlData;
	    	$success = "Given API data (Secret Access Key and Access Key ID) for $amz_marketplace seems to work porperly.<br />If not saved yet you can/should do this now.";
	    }



		}
 		$this->json = json_encode(array("success" => $success, "error" => $error ? implode("<br />",$error) : false,
 			"xmlError" => $xmlError, "response" => $response
 			));
	}






	//update amazon config data
	function updateAmzConfig($data) {
		$error = array();
		$success = false;
		if (!$this->auth) {
			$error[] = "Please (re-)autenticate.";
		}
		if (!$error) {
			$amz_marketplace = trim($data["amz_marketplace"]);
			$amz_tag = trim($data["amz_tag"]);
			$amz_access_key = trim($data["amz_access_key"]);
			$amz_secret_key = trim($data["amz_secret_key"]);

			if (!$amz_marketplace) $error[] = "Please select your amazon marketplace.";
			if (!$amz_tag) $error[] = "Please type in your amazon associate tag.";
			if (!$amz_access_key) $error[] = "Please type in the access key id.";
			if (!$amz_secret_key) $error[] = "Please type in the secret key.";
		}
		if (!$error) {
			$sql = "update `" . $this->tablePrefix . "config` set 
				amz_marketplace = " . $this->dbquote($amz_marketplace) . ", 
				amz_tag = " . $this->dbquote($amz_tag) . ", 
				amz_access_key = " . $this->dbquote($amz_access_key) . ", 
				amz_secret_key = " . $this->dbquote($amz_secret_key) . " 
				where id = '" . intval($this->configId) . "'";
			$this->dbquery($sql);
			$success = true;		

		}
 		$this->json = json_encode(array("success" => $success, "error" => $error ? implode("<br />",$error) : false,
 			"amz_tag" => $amz_tag, "amz_access_key" => $amz_access_key, "amz_secret_key" => str_repeat("&#9679;",20)
 			));
	}

	//update site config data
	function updateSiteConfig($data) {
		$error = array();
		$success = false;
		if (!$this->auth) {
			$error[] = "Please (re-)autenticate.";
		}
		if (!$error) {


			$lang = trim($data["lang"]);
			$site_title = strip_tags(trim($data["site_title"]));
			$site_author = strip_tags(trim($data["site_author"]));
			$contact_mail = strtolower(trim($data["contact_mail"]));
			$categories_in_navbar = intval($data["categories_in_navbar"]);
			$meta_description = strip_tags(trim($data["meta_description"]));
			$meta_keywords = strip_tags(trim($data["meta_keywords"]));
			$affiliate_username=strip_tags(trim($data["affiliate_username"]));

			if (!$lang) $error[] = "Please select the front end site language.";
			if (!$contact_mail) $error[] = "Please type in a contact email address.";
			elseif (!$this->is_email($contact_mail)) {
				$error[] = "The email address you typed in seems to be incorrect.";
			}
		}
		if (!$error) {
			$sql = "update `" . $this->tablePrefix . "config` set 
				lang = " . $this->dbquote($lang) . ", 
				affiliate_username=".$this->dbquote($affiliate_username).",
				site_title = " . $this->dbquote($site_title) . ", 
				site_author = " . $this->dbquote($site_author) . ", 
				meta_description = " . $this->dbquote($meta_description) . ", 
				meta_keywords = " . $this->dbquote($meta_keywords) . ", 
				contact_mail = " . $this->dbquote($contact_mail) . ", 
				categories_in_navbar = '$categories_in_navbar' 
				where id = '" . intval($this->configId) . "'";
			$this->dbquery($sql);
			$success = true;		

		}
 		$this->json = json_encode(array("success" => $success, "error" => $error ? implode("<br />",$error) : false,
 			"site_title" => $site_title, "site_author" => $site_author,"affiliate_username"=>$affiliate_username, "meta_description" => $meta_description, "meta_keywords" => $meta_keywords, "contact_mail" => $contact_mail
 			));
	}

	//update amazon search results data
	function updateSrchConfig($data) {
		$error = array();
		$success = false;
		if (!$this->auth) {
			$error[] = "Please (re-)autenticate.";
		}
		if (!$error) {
			$show_amz_search_results = intval($data["show_amz_search_results"]);
			$link_amz_search_results_to_amz = intval($data["link_amz_search_results_to_amz"]);
			$link_to_more_results_on_amz = intval($data["link_to_more_results_on_amz"]);

			$show_amz_search_results = $show_amz_search_results == 1 ?: 0;
			$link_amz_search_results_to_amz = $link_amz_search_results_to_amz == 1 ?: 0;
			$link_to_more_results_on_amz = $link_to_more_results_on_amz == 1 ?: 0;

		}
		if (!$error) {
			$sql = "update `" . $this->tablePrefix . "config` set 
				show_amz_search_results = '$show_amz_search_results', 
				link_amz_search_results_to_amz = '$link_amz_search_results_to_amz', 
				link_to_more_results_on_amz = '$link_to_more_results_on_amz'
				where id = '" . intval($this->configId) . "'";
			$this->dbquery($sql);
			$success = true;		

		}
 		$this->json = json_encode(array("success" => $success, "error" => $error ? implode("<br />",$error) : false
 			));
	}

	//update amazon suggestions default values
	function updateSuggestionConfig($data) {
		$error = array();
		$success = false;
		if (!$this->auth) {
			$error[] = "Please (re-)autenticate.";
		}
		if (!$error) {
			$default_products_show_amz_suggestions = intval($data["default_products_show_amz_suggestions"]);
			$default_products_link_amz_suggestions_to_amz = intval($data["default_products_link_amz_suggestions_to_amz"]);

			$default_products_show_amz_suggestions = $default_products_show_amz_suggestions == 1 ?: 0;
			$default_products_link_amz_suggestions_to_amz = $default_products_link_amz_suggestions_to_amz == 1 ?: 0;
		}
		if (!$error) {
			$sql = "update `" . $this->tablePrefix . "config` set 
				default_products_show_amz_suggestions = '$default_products_show_amz_suggestions', 
				default_products_link_amz_suggestions_to_amz = '$default_products_link_amz_suggestions_to_amz'
				where id = '" . intval($this->configId) . "'";
			$this->dbquery($sql);
			$success = true;		

		}
 		$this->json = json_encode(array("success" => $success, "error" => $error ? implode("<br />",$error) : false
 			));
	}


	//update show amazon suggestions for specific items
	function updateShowAmazonSuggestions($data) {
		$error = array();
		$sourcesSql = null;
		$success = false;
		if (!$this->auth) {
			$error[] = "Please (re-)autenticate.";
		}
		if (!$error) {
			if (!$data["catIds"] && !$data["sources"]) {
				$error[] = "Please select some items to modify.";
			}
		}
		if (!$error) {
			$show_amz_suggestions = intval($data["show_amz_suggestions"]);
			$show_amz_suggestions = $show_amz_suggestions == 1 ?: 0;
		}
		if (!$error) {
			if ($data["sources"]) {
				$sourceValues = array();
				//don't trust the admin :)
				foreach ($data["sources"] as $source) {
					if ($source) {
						$sourceValues[] = $this->dbquote($source);
					}
				}
				if ($sourceValues) {
					$sourcesSql = "source in (" . implode(",",$sourceValues) . ")";
				}
			}

			//all items?
			if ($data["catIds"] && in_array("all",$data["catIds"])) {
				$sql = "update `" . $this->tablePrefix . "products` set show_amz_suggestions = '$show_amz_suggestions'";
				$this->dbquery($sql);
			} else {
				if ($data["catIds"]) {
					if ($sourcesSql) {
						$sourcesSql = " or $sourcesSql";
					}
					foreach ($data["catIds"] as $catId) {
						if ($catId) {
							$sql = "update `" . $this->tablePrefix . "products` set show_amz_suggestions = '$show_amz_suggestions' 
								where id in (select product_id from `" . $this->tablePrefix . "category_products` where category_id = '" . intval($catId) . "') $sourcesSql";
							$this->dbquery($sql);
						} else {
							$sql = "update `" . $this->tablePrefix . "products` set show_amz_suggestions = '$show_amz_suggestions' 
								where id not in (select product_id from `" . $this->tablePrefix . "category_products`) $sourcesSql";
							$this->dbquery($sql);
						}
					}
				} elseif ($data["sources"] && $sourcesSql) {
					$sql = "update `" . $this->tablePrefix . "products` set show_amz_suggestions = '$show_amz_suggestions' where $sourcesSql";
					$this->dbquery($sql);
				}
			}
			$success = true;
		}
 		$this->json = json_encode(array("success" => $success, "error" => $error ? implode("<br />",$error) : false
 			));
	}

	// update Amazon Suggestion linkings for specified products
	function updateLinkAmazonSuggestions($data) {
		$error = array();
		$success = false;
		if (!$this->auth) {
			$error[] = "Please (re-)autenticate.";
		}
		if (!$error) {
			if (!$data["catIds"] && !$data["sources"]) {
				$error[] = "Please select some items to modify.";
			}
		}
		if (!$error) {
			$link_amz_suggestions_to_amz = intval($data["link_amz_suggestions_to_amz"]);
			$link_amz_suggestions_to_amz = $link_amz_suggestions_to_amz == 1 ?: 0;
		}
		if (!$error) {
			if ($data["sources"]) {
				$sourceValues = array();
				//don't trust the admin :)
				foreach ($data["sources"] as $source) {
					if ($source) {
						$sourceValues[] = $this->dbquote($source);
					}
				}
				if ($sourceValues) {
					$sourcesSql = "source in (" . implode(",",$sourceValues) . ")";
				}
			}

			//all items?
			if ($data["catIds"] && in_array("all",$data["catIds"])) {
				$sql = "update `" . $this->tablePrefix . "products` set link_amz_suggestions_to_amz = '$link_amz_suggestions_to_amz'";
				$this->dbquery($sql);
			} else {
				if ($data["catIds"]) {
					if ($sourcesSql) {
						$sourcesSql = " or $sourcesSql";
					}
					foreach ($data["catIds"] as $catId) {
						if ($catId) {
							$sql = "update `" . $this->tablePrefix . "products` set link_amz_suggestions_to_amz = '$link_amz_suggestions_to_amz' 
								where id in (select product_id from `" . $this->tablePrefix . "category_products` where category_id = '" . intval($catId) . "') $sourcesSql";
							$this->dbquery($sql);
						} else {
							$sql = "update `" . $this->tablePrefix . "products` set link_amz_suggestions_to_amz = '$link_amz_suggestions_to_amz' 
								where id not in (select product_id from `" . $this->tablePrefix . "category_products`) $sourcesSql";
							$this->dbquery($sql);
						}
					}
				} elseif ($data["sources"] && $sourcesSql) {
					$sql = "update `" . $this->tablePrefix . "products` set link_amz_suggestions_to_amz = '$link_amz_suggestions_to_amz' where $sourcesSql";
					$this->dbquery($sql);
				}
			}
			$success = true;
		}
 		$this->json = json_encode(array("success" => $success, "error" => $error ? implode("<br />",$error) : false
 			));
	}

	// remove items - multi product operations
	function removeMultiSelectedItems($data) {
		$error = array();
		$success = false;
		if (!$this->auth) {
			$error[] = "Please (re-)autenticate.";
		}
		if (!$error) {
			if (!$data["catIds"] && !$data["sources"]) {
				$error[] = "Please select some items to modify.";
			}
		}
		if (!$error) {
			if ($data["sources"]) {
				$sourceValues = array();
				//don't trust the admin :)
				foreach ($data["sources"] as $source) {
					if ($source) {
						$sourceValues[] = $this->dbquote($source);
					}
				}
				if ($sourceValues) {
					$sourcesSql = "source in (" . implode(",",$sourceValues) . ")";
				}
			}

			//all items?
			if ($data["catIds"] && in_array("all",$data["catIds"])) {
				$sql = "select id from `" . $this->tablePrefix . "products`";
			} else {
				if ($data["catIds"]) {
					if ($sourcesSql) {
						$sourcesSql = " or $sourcesSql";
					}
					foreach ($data["catIds"] as $catId) {
						if ($catId) {
							$sql = "select id from `" . $this->tablePrefix . "products` where id in (select product_id from `" . $this->tablePrefix . "category_products` where category_id = '" . intval($catId) . "') $sourcesSql";
						} else {
							$sql = "select id from `" . $this->tablePrefix . "products` where id not in (select product_id from `" . $this->tablePrefix . "category_products`) $sourcesSql";
						}
					}
				} elseif ($data["sources"] && $sourcesSql) {
					$sql = "select id from `" . $this->tablePrefix . "products` where $sourcesSql";
				}
			}
			if ($sql) {
				$res = $this->dbquery($sql);
				while ($row = mysqli_fetch_object($res)) {
					$this->removeItem($row->id);
				}
					sleep(4);
				
				$success = true;
			}
		}
 		$this->json = json_encode(array("success" => $success, "error" => $error ? implode("<br />",$error) : false
 			));
	}



	//update/create product category
	function editCategory($data) {
		$error = array();
		$success = false;
		$newCatId = 0;
		if (!$this->auth) {
			$error[] = "Please (re-)autenticate.";
		}
		if (!$error) {
			$catId = intval($data["catId"]);
			$parent = intval($data["parent"]);
			$status = intval($data["status"]);
			$descr = strip_tags(trim($data["descr"]));
			$short_intro = strip_tags(trim($data["short_intro"]));
			$page_title = strip_tags(trim($data["page_title"]));
			$page_descr = strip_tags(trim($data["page_descr"]));
			$headline = trim($data["headline"]);
			$long_intro = trim($data["long_intro"]);

			if (!$descr) $error[] = "Please type in name for your category.";
		}
		if (!$error) {

			$add2sql = "descr = " . $this->dbquote($descr) . ", 
				short_intro = " . $this->dbquote($short_intro) . ", 
				page_title = " . $this->dbquote($page_title) . ", 
				page_descr = " . $this->dbquote($page_descr) . ", 
				headline = " . $this->dbquote($headline) . ", 
				long_intro = " . $this->dbquote($long_intro) . ", 
				status = '$status',
				parent = '$parent'";
			if ($catId) { 
				//check current parent for repos issues
				$catData = $this->getCategoryData($catId,false);
				if ($catData->parent != $parent) {
					$add2sql .= ", pos = 999";
				}
				$sql = "update `" . $this->tablePrefix . "categories` set 
					$add2sql, 
					edited = now() where id = '$catId'";
				$this->dbquery($sql);
				$this->reposCategories($catData->parent);
				$success = "The category has been updated successfully.";		
			} else {
				$sql = "insert into `" . $this->tablePrefix . "categories` set 
					$add2sql, 
					created = now()";
				$this->dbquery($sql);
				$newCatId = mysqli_insert_id($this->db);
				$success = "The category has been created successfully.";		
			}
			$this->reposCategories($parent);

		}
 		$this->json = json_encode(array("success" => $success, "error" => $error ? implode("<br />",$error) : false,
 			"descr" => $descr, "newCatId" => $newCatId, "url" => $this->instDir . "/cat" . ($catId?:$newCatId) . "/" . $this->tidyup($descr) . ".html"
 			));
	}

	//to reorder after deletion, repositioning or adding of categories
	function reposCategories($parent=0) {
		$parent = intval($parent);
		$pos = 1;
		$sql = "select * from `" . $this->tablePrefix . "categories` where parent = '$parent' order by pos";
		$res = $this->dbquery($sql);
		while ($row = mysqli_fetch_object($res)) {
			$sql = "update `" . $this->tablePrefix . "categories` set pos = '" . $pos++ . "' where id = '" . $row->id . "'";
			$this->dbquery($sql);
		}
	}

	//order single categories a level up or down
	function positionCategory($catId,$dir,$parent) {
		$catId = intval($catId);
		$parent = intval($parent);
		if (!$catId) return false;
		// we need the parent
		$catData = $this->getCategoryData($catId,false);
		if (!$catData) return false;
		if ($dir == "up") {
			$sql = "update `" . $this->tablePrefix . "categories` set pos = pos - 1 where id = '$catId'";
			$this->dbquery($sql);
			//repos lower positioned one
			$sql = "update `" . $this->tablePrefix . "categories` set pos = pos + 1 where pos = '" . ($catData->pos - 1) . "' and parent = '$parent' and id != '$catId'";
			$this->dbquery($sql);
		} elseif ($dir == "down") {
			$sql = "update `" . $this->tablePrefix . "categories` set pos = pos + 1 where id = '$catId'";
			$this->dbquery($sql);
			//repos higher positioned one
			$sql = "update `" . $this->tablePrefix . "categories` set pos = pos - 1 where pos = '" . ($catData->pos + 1) . "' and parent = '$parent' and id != '$catId'";
			$this->dbquery($sql);
		}
		//for the case of a mistake
		$this->reposCategories($catData->parent);

 		$this->json = json_encode(array("success" => true));

	}



	//change status of categories
	function changeCategoryStatus($catId,$status) {
		$catId = intval($catId);
		$status = intval($status);
		$status = $status == 1 ?: 0;
		if (!$catId) return false;
		$sql = "update `" . $this->tablePrefix . "categories` set status = '$status' where id = '$catId'";
		$this->dbquery($sql);
 		$this->json = json_encode(array("success" => true, "status" => $status));
	}



	//change top-cat status of categories
	function changeTopCatState($catId,$top_cat) {
		$catId = intval($catId);
		$top_cat = intval($top_cat);
		$top_cat = $top_cat == 1 ?: 0;
		if (!$catId) return false;
		$sql = "update `" . $this->tablePrefix . "categories` set top_cat = '$top_cat' where id = '$catId'";
		$this->dbquery($sql);
 		$this->json = json_encode(array("success" => true, "top_cat" => $top_cat));
	}


	function importASINlist($asinList,$cat_ids=false,$source="admin-import") {
		// if (!$this->auth) return false; // needed for internal suggestion linking or amazon search results linking
		$error = false;
		$product_id = 0;
		$asinList = trim($asinList);


		if (!$asinList) {
			$error = "Please insert some ASINs.";
		}

		if (is_array($cat_ids)) {
			//check available categories
			list($a,$b,$categories) = $this->getCategories();
			if (!$categories) {
				$cat_ids = array();
			} else {
				$cat_ids_tmp = array();
				foreach ($categories as $cat) {
					if (in_array($cat->id,$cat_ids)) {
						$cat_ids_tmp[] = $cat->id;
					}
				}
				$cat_ids = $cat_ids_tmp;
			}
		}

		if (!$error) {
			$dupes = $added = 0;
			$invalid = array();
			$lines = preg_split("/(\r\n|\n|\r)/",$asinList);
			foreach ($lines as $line) {
				$asin = preg_replace('/[^a-zA-Z0-9]/','', trim($line));
				if (strlen($asin) > 7 && strlen($asin) < 20) { //to avoid any unwanted strings...
					//already imported??
					$sql = "select * from `" . $this->tablePrefix . "products` where asin = " . $this->dbquote($asin);
					$res = $this->dbquery($sql);
					if (!mysqli_num_rows($res)) {
						//insert into system
						$sql = "insert into `" . $this->tablePrefix . "products` set 
							asin = " . $this->dbquote($asin) . ", 
							created = now(), 
							source = " . $this->dbquote($source). ", 
							show_amz_suggestions = '" . $this->config->default_products_show_amz_suggestions . "',
							link_amz_suggestions_to_amz = '" . $this->config->default_products_link_amz_suggestions_to_amz . "'";
						$this->dbquery($sql);
						$product_id = mysqli_insert_id($this->db);
						if ($cat_ids) {
							$product_id = mysqli_insert_id($this->db);
							foreach ($cat_ids as $catId) {
								$sql = "insert into `" . $this->tablePrefix . "category_products` set product_id = '$product_id', category_id = '$catId', created = now()";
								$this->dbquery($sql);
							}
						}
						$added++;
					} else {
						$dupes++;
					}
				} else {
					$invalid[] = $asin;
				}
			}
		}
		if ($dupes || $added || $invalid) {
			$this->success = "The data transmitted has been processed successfully.<ul class='mb-0'>
				<li>newly added: <strong>$added</strong> items</li/>
				<li>already imported: <strong>$dupes</strong> items</li/>
				<li class='text-danger'>invalid ASINs: <strong>" . count($invalid) . "</strong>" . ($invalid ? " (" . implode(", ",$invalid) . ")" : "") . "</li/></ul>";
			return $product_id;
		}
		if ($error) {
			$this->error = $error;
		}
	}


	function removeItem($itemId) {
		$error = $success = false;
		$itemId = intval($itemId);
		if (!$itemId) {
			$error = "Unsufficient data provided.";
		}
		if (!$this->auth) {
			$error = "Please (re)authenticate.";
		}

		if (!$error) {
			// we need asin for products
			$sql = "select asin from `" . $this->tablePrefix . "products` where id = '$itemId'";
			$res = $this->dbquery($sql);
			$row = mysqli_fetch_object($res);
			$asin = $row->asin;
			//remove item
			$sql = "delete from `" . $this->tablePrefix . "products` where id = '$itemId'";
			$this->dbquery($sql);
			//remove assignments
			$sql = "delete from `" . $this->tablePrefix . "category_products` where product_id = '$itemId'";
			$this->dbquery($sql);
			//remove images
			if ($asin) {
				$sql = "delete from `" . $this->tablePrefix . "image_sets` where asin = " . $this->dbquote($asin);
				$this->dbquery($sql);
				//remove cached XML files
				$xmlFiles = glob($this->xmlCacheDir . "*/" . $asin . "*xml");
				foreach ($xmlFiles as $file) { 
		      if (preg_match("/^" . $asin . "(_err)?\.xml$/",$file)) {
		      	unlink($file);
		      }
		   	} 
			}
			//remove reviews
			$sql = "delete from `" . $this->tablePrefix . "editorial_reviews` where asin = " . $this->dbquote($asin);
			$this->dbquery($sql);
			//remove variations
			$sql = "delete from `" . $this->tablePrefix . "variations` where parent_asin = " . $this->dbquote($asin) . " or asin = " . $this->dbquote($asin);
			$this->dbquery($sql);
			

			$success = true;
		}
 		$this->json = json_encode(array("success" => $success, "error" => $error));
	}


	function removeItems($itemIds) {
		$error = $success = false;
		if (!$itemIds) {
			$error = "Unsufficient data provided.";
		}
		if (!$this->auth) {
			$error = "Please (re)authenticate.";
		}

		if (!$error) {
			//let's use the removeItem() method
			foreach ($itemIds as $itemId) {
				$this->removeItem(intval($itemId));
			}
			$success = true;
		}
 		$this->json = json_encode(array("success" => $success, "error" => $error));
	}


	function readXMLfiles($asin) {
		$asin = trim($asin);
		$error = $bbMessage = $bbTabs = null;
		$xmlFiles = array();
		if (!$asin) {
			$error = "Please provide an ASIN.";
		}
		if (!$error) {
			//check for files
			if (file_exists($this->xmlCacheDir . $this->config->amz_marketplace . "/" . $asin . ".xml")) {
				$fileContent = file_get_contents($this->xmlCacheDir . $this->config->amz_marketplace . "/" . $asin . ".xml");
				$fileContent = preg_replace('/' . $this->config->amz_access_key . '/i', '[blackened content]', $fileContent);
				$fileContent = preg_replace('/Name="Signature" Value="[^"]+"/', 'Name="Signature" Value="[blackened content]"', $fileContent);

				$x = new DOMDocument;
				$x->preserveWhiteSpace = false;
				$x->formatOutput = true;
				$x->loadXML($fileContent);
				$cleanXML = $x->saveXML();

				$cleanXML = htmlentities($cleanXML);
				$cleanXML = preg_replace('/(\[blackened content\])/', '<span class="bg-dark">$1</span>', $cleanXML);


				$xmlFiles[] = array("filename" => $asin . ".xml",
														"filedate" => date("Y M jS g:i A",filemtime($this->xmlCacheDir . $this->config->amz_marketplace . "/" . $asin . ".xml")),
														"filecontent" => $cleanXML);
			}
			if (file_exists($this->xmlCacheDir . $this->config->amz_marketplace . "/" . $asin . "_err.xml")) {
				$fileContent = file_get_contents($this->xmlCacheDir . $this->config->amz_marketplace . "/" . $asin . "_err.xml");
				$fileContent = preg_replace('/' . $this->config->amz_access_key . '/i', '[blackened content]', $fileContent);
				$fileContent = preg_replace('/Name="Signature" Value="[^"]+"/', 'Name="Signature" Value="[blackened content]"', $fileContent);

				$x = new DOMDocument;
				$x->preserveWhiteSpace = false;
				$x->formatOutput = true;
				$x->loadXML($fileContent);
				$cleanXML = $x->saveXML();

				$cleanXML = htmlentities($cleanXML);
				$cleanXML = preg_replace('/(\[blackened content\])/', '<span class="bg-dark">$1</span>', $cleanXML);

				$xmlFiles[] = array("filename" => $asin . "_err.xml",
														"filedate" => date("Y M jS g:i A",filemtime($this->xmlCacheDir . $this->config->amz_marketplace . "/" . $asin . "_err.xml")),
														"filecontent" => $cleanXML);
			} 
		}
		if (!$xmlFiles) {
			$error = "There are no XML files for this ASIN stored on the server.";
		} else {
			$bbMessage = '<ul class="nav nav-tabs">';
			'<div class="tab-content" id="myTabContent">
			  <div class="tab-pane fade show active" id="home" role="tabpanel" aria-labelledby="home-tab">...</div>
			  <div class="tab-pane fade" id="profile" role="tabpanel" aria-labelledby="profile-tab">...</div>
			  <div class="tab-pane fade" id="contact" role="tabpanel" aria-labelledby="contact-tab">...</div>
			</div>';

			$i = 0;
			$activeClass = "active";
			foreach ($xmlFiles as $xmlFile) {
				extract($xmlFile);
				$i++;
				$bbMessage .= '<li class="nav-item">
			    <a class="nav-link ' . $activeClass . ' data-toggle="tab" href="#file-' . $i . '">' . $filename . '<br /><small>' . $filedate . '</small></a>
			 	</li>';
			 	$bbTabs .= '<div class="tab-pane fade show ' . $activeClass . '" id="file-' . $i . '"><pre class="small code striped-lines">' . $filecontent . '</pre></div>';
				$activeClass = null;
			}
			$bbMessage .= '</ul>
				<div class="tab-content" id="myTabContent">' . $bbTabs . '</div>';
			// $bbMessage = implode("<hr />",$xmlFiles);
		}

 		$this->json = json_encode(array("error" => $error, "bbMessage" => $bbMessage));

	}


	function setItemCat($itemId, $category_id, $todo) {
		$success = $error = false;
		$catDescr = array();
		$itemId = intval($itemId);
		$category_id = intval($category_id);
		if (!$itemId) {
			$error = "Unsufficient data provided.";
		}
		if (!$this->auth) {
			$error = "Please (re)authenticate.";
		}
		if (!$error) {
			//remove first - to avoid dupes
			$sql = "delete from `" . $this->tablePrefix . "category_products` where product_id = '$itemId' and category_id = '$category_id'";
			$this->dbquery($sql);
			if ($todo == "add") {
				$sql = "insert into `" . $this->tablePrefix . "category_products` set product_id = '$itemId', category_id = '$category_id', created = now()";
				$this->dbquery($sql);
			}
			//get current category descriptions
			$sql = "select descr from `" . $this->tablePrefix . "categories` where id in (select category_id from `" . $this->tablePrefix . "category_products` where product_id = '$itemId')";
			$res = $this->dbquery($sql);
			if (mysqli_num_rows($res)) {
				while ($row = mysqli_fetch_object($res)) {
					$catDescr[] = $row->descr;
				}
			}
			$success = "The item has been updated successfully.";
		}
 		$this->json = json_encode(array("success" => $success, "error" => $error, "catDescr" => $catDescr ? implode(", ", $catDescr) : false));
	}


	function uncatItems($itemIds) {
		$error = $success = false;
		if (!$itemIds) {
			$error = "Unsufficient data provided.";
		}
		if (!$this->auth) {
			$error = "Please (re)authenticate.";
		}

		if (!$error) {
			foreach ($itemIds as $itemId) {
				$sql = "delete from `" . $this->tablePrefix . "category_products` where product_id = '" . intval($itemId) . "'";
				$this->dbquery($sql);
			}
			$success = true;
		}
 		$this->json = json_encode(array("success" => $success, "error" => $error));
	}


	function setItemsCat($itemIds, $category_id) {
		$success = $error = false;
		$catDescrs = array();
		$category_id = intval($category_id);
		if (!$itemIds) {
			$error = "Unsufficient data provided.";
		}
		if (!$this->auth) {
			$error = "Please (re)authenticate.";
		}

		if (!$error) {
			foreach ($itemIds as $itemId) {
				$itemId = intval($itemId);
				$catDescr = array();
				if ($itemId) {
					//remove first - to avoid dupes
					$sql = "delete from `" . $this->tablePrefix . "category_products` where product_id = '$itemId' and category_id = '$category_id'";
					$this->dbquery($sql);
					//(re)add
					$sql = "insert into `" . $this->tablePrefix . "category_products` set product_id = '$itemId', category_id = '$category_id', created = now()";
					$this->dbquery($sql);
					//get current category descriptions
					$sql = "select descr from `" . $this->tablePrefix . "categories` where id in (select category_id from `" . $this->tablePrefix . "category_products` where product_id = '$itemId')";
					$res = $this->dbquery($sql);
					if (mysqli_num_rows($res)) {
						while ($row = mysqli_fetch_object($res)) {
							$catDescr[] = $row->descr;
						}
					}
					$catDescrs[$itemId] = implode(", ", $catDescr);
				}
			}
			$success = "The items have been categorized successfully.";
		}
 		$this->json = json_encode(array("success" => $success, "error" => $error, "catDescrs" => $catDescrs));
	}



	function importSingleAsin($asin,$cat_ids=false) {
		$error = $success = false;
		$asin = trim($asin);
		if (!$asin) {
			$error = "Unsufficient data provided.";
		}
		if (!$this->auth) {
			$error = "Please (re)authenticate.";
		}

		if (!$error) {
			$this->importASINlist($asin,$cat_ids);
			$error = $this->error;
			if (!$error) {
				$success = true;
			}
		}
 		$this->json = json_encode(array("success" => $success, "error" => $error));
	}


	function searchAmazonByKeyword($srch,$searchindex="All",$link_amz_search_results_to_amz = -1) {
		$srch = trim($srch);
		$results = $searchResults = $asins = $asinsInDb = array();
		$XMLurl = $this->amzGetXMLUrl("keywords",array("srch" => $srch, "searchindex" => $searchindex));

		//CURL Version
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $XMLurl);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    // get the result of http query
    $output = curl_exec($ch);
    curl_close($ch);
    
    if (!preg_match('/^</',$output)) {
      $this->error = "Data of $url not completed loaded.<br />
        Output:<pre>$output</pre>";
    } else {
      $xmlData = simplexml_load_string($output);
    }


    // echo "<pre>" . print_r($xmlData,true) . "</pre>";


    $items = (array)$xmlData->Items;
    // echo "<pre>" . print_r($items["Item"],true) . "</pre>";
    if (isset($items["Item"])) {
	    foreach ($items["Item"] as $item) {
	    	if (!$item->ItemAttributes->Title) continue;
	    	$itemData["asin"] = strval($item->ASIN);
	    	$itemData["item_url"] = strval($item->DetailPageURL);
	    	$itemData["image_url"] = strval($item->LargeImage->URL);
	    	$itemData["image_url_medium"] = preg_replace('/\.jpg$/',"._SL200_.jpg",strval($item->LargeImage->URL));
	    	$itemData["title"] = strval($item->ItemAttributes->Title);
	    	$asins[] = strval($item->ASIN);
	    	$results[] = (object) $itemData;

	    	//add to store db - if adding is enabled by internal search results item linking
	      if ($link_amz_search_results_to_amz == 0) {
	      	$this->importASINlist($itemData["asin"],false,"search-results");
	      }
	    }
	  }
    //check if in db
    foreach ($asins as $asin) {
    	$sql = "select id from `" . $this->tablePrefix . "products` where asin = " . $this->dbquote($asin);
    	$res = $this->dbquery($sql);
    	if (mysqli_num_rows($res)) $asinsInDb[] = $asin;
    }

    $searchResults = (object)array("results" => $results,
				    											 "MoreSearchResultsUrl" => $items["MoreSearchResultsUrl"],
				    											 "TotalResults" => $items["TotalResults"],
				    											 "asinsInDb" => $asinsInDb
				    											 );

    // echo "<pre>" . print_r($searchResults,true) . "</pre>";

    return $searchResults;
	}




	/**** import methods/functions ****/
	// limit ... how much items should be feched in one round ... depending on your given time frames - 50items/5minutes are fine
	function amzHandleFirstImports($limit = 50) {
		$limit = intval($limit);
		if (!$limit) $limit = 50;
		$sql = "select * from `" . $this->tablePrefix . "products` where imported is NULL and import_failed is NULL order by id limit $limit";
		$res = $this->dbquery($sql);
		while ($row = mysqli_fetch_object($res)) {
			echo $row->asin . "<br />\n";
			if (!$this->amzFirstItemUpdate($row->asin)) {
				$sql = "update `" . $this->tablePrefix . "products` set import_failed = now() where asin = " . $this->dbquote($asin);
				$this->dbquery($sql);
			}
			sleep(5);
		}
	}

	/*** get number of items ready to import ***/
	function getNumberOfNotImportedDataItems() {
		$sql = "select id from `" . $this->tablePrefix . "products` where imported is NULL and import_failed is NULL";
		$res = $this->dbquery($sql);
		return intval(mysqli_num_rows($res));
	}

	/*** single asin for the grab now import ***/
	function getAnyASINReadyToImport() {
		$sql = "select asin from `" . $this->tablePrefix . "products` where imported is NULL and import_failed is NULL limit 1";
		$res = $this->dbquery($sql);
		if (mysqli_num_rows($res)) {
			$row = mysqli_fetch_object($res);
			return $row->asin;
		} else {
			$this->json = json_encode(array(
				"error" => "There are no items ready to import found in the database.",
				"noAsin" => 1
			));
			return false;
		}

	}


	function amzQuickASINimport($asin,$cat_ids) {
		if (!$asin) return false;
		$product_id = $this->importASINlist($asin,$cat_ids);
		if ($product_id) {
			if (!$this->amzFirstItemUpdate($asin)) {
				$sql = "update `" . $this->tablePrefix . "products` set import_failed = now() where asin = " . $this->dbquote($asin);
				$this->dbquery($sql);
			} else {
				return $product_id;
			}
		}
	}

	function amzImportOnCall($asin) {
		if (!$asin) return false;
		if (!$this->amzFirstItemUpdate($asin)) {
			$sql = "update `" . $this->tablePrefix . "products` set import_failed = now() where asin = " . $this->dbquote($asin);
			$this->dbquery($sql);
			return false;
		} else {
			return true;
		}
	}




	/**** update possible expirinhg review iframe urls ****/
	function amzUpdateReviewsIframeUrl($asin) {
		$asin = trim($asin);
		$xmlError = null;
		if (!$asin) return false;
		$expiringHours = 23;
		$reviews_iframe_url = null;
		$has_reviews = 0;
		$sql = "select * from `" . $this->tablePrefix . "products` where asin = " . $this->dbquote($asin) . " and date_add(reviews_iframe_url_last_update, interval " . $expiringHours . " hour) <= now() and imported is not null";
		$res = $this->dbquery($sql);
		if (mysqli_num_rows($res)) {
			$row = mysqli_fetch_object($res);

			$XMLurl = $this->amzGetXMLUrl("reviews",array("ASIN" => $asin));

			//CURL Version
	    $ch = curl_init();
	    curl_setopt($ch, CURLOPT_URL, $XMLurl);
	    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	    // get the result of http query
	    $output = curl_exec($ch);
	    curl_close($ch);

	    if (!preg_match('/^</',$output)) {
	      $this->error = "Data of $url not completed loaded.<br />
	        Output:<pre>$output</pre>";
	    } else {
	      $xmlData = simplexml_load_string($output);
	    }

	    // echo "<pre>". print_r($xmlData->Items->Request->Errors,true) . "</pre><hr />";
	    if ($xmlData->Items->Request->Errors) {
	    	$xmlError = "----ERROR ASIN:$ASIN<br /><pre>" . print_r($xmlData->Items,true) . "</pre><hr />\n";
	    }
	    if (!$xmlError) {
		    $reviews_iframe_url = $this->amzKeysToPlaceholders(strval($xmlData->Items->Item->CustomerReviews->IFrameURL));
			  $has_reviews = strval($xmlData->Items->Item->CustomerReviews->HasReviews) == "true" ? 1 : 0;
	    }

		  //let's update the database
		  $sql = "update `" . $this->tablePrefix . "products` set 
		  	reviews_iframe_url = " . $this->dbquote($reviews_iframe_url) . ",
		  	has_reviews = '$has_reviews',
		  	reviews_iframe_url_last_update = now()
		  	where id = '" . $row->id . "'";
		  $this->dbquery($sql);
		  //return new url and has_reviews info
		  return array($has_reviews,$reviews_iframe_url);
		}
		return false;
	}


	//get rating for item :)
	function fetchItemRating($asin) {
		if (!$asin) return false;
		$ratingFetchUrl = "https://[AMZ-URL]/gp/customer-reviews/widgets/average-customer-review/popover/ref=dpx_acr_pop_?contextId=dpx&asin=[ASIN]";

		$amz_url = preg_replace('/^[a-z]+\./', 'www.', $this->config->amz_marketplace);

		$ratingFetchUrl = str_replace(
			array("[AMZ-URL]","[ASIN]"), 
			array($amz_url,$asin),
			$ratingFetchUrl
		);

		//CURL Version
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $ratingFetchUrl);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    // get the result of http query
    $output = curl_exec($ch);
    curl_close($ch);

    if (!preg_match('/histogramTable/',$output)) {
    	return false;
    }

    $output = trim(strip_tags($output));
    $output = preg_replace("/(^[\r\n]*|[\r\n]+)[\s\t]*[\r\n]+/", "\n",$output);
    $output = preg_replace("/^\s+/m", "",$output);
    $output = preg_replace("/^\s*(\d+\s\w+)\n\s*/m", '$1 ',$output);

    $rating = floatval($output);

		preg_match_all('/\d+/',array_pop(preg_split('/[\n\r]/',$output)),$matches);
		$reviews = intval($matches[0][0]);

    return array($rating,$output,$reviews);

	}


	/**** update item ratings ****/
	function updateItemRating($asin,$expiringHours = 12) {
		$asin = trim($asin);
		if (!$asin) return false;
		$sql = "select * from `" . $this->tablePrefix . "products` where asin = " . $this->dbquote($asin) . " and (last_rating_update is null or date_add(last_rating_update, interval " . $expiringHours . " hour) <= now()) and imported is not null";
		$res = $this->dbquery($sql);
		if (mysqli_num_rows($res)) {
			$row = mysqli_fetch_object($res);

			$add2sql = null;

			$ratingData = $this->fetchItemRating($asin);
			$rating = 0;

			if ($ratingData) {
				$add2sql = ", rating = " . floatval($ratingData[0]) . ",
					rating_dom = " . $this->dbquote($ratingData[1]) . ",
					reviews = " . $this->dbquote($ratingData[2]);
					$rating = $ratingData[0];
					$reviews = $ratingData[2];
			} 
		  //let's update the database
		  $sql = "update `" . $this->tablePrefix . "products` set 
				last_rating_update = now() $add2sql
		  	where id = '" . $row->id . "'";
		  $this->dbquery($sql);
		  //return new current ratingg
		  return array($rating,$reviews);
		}
	}

	function drawRatingStars($rating) {
		if (!$rating) return false;
		$fulls = floor($rating);
		$half = $rating - $fulls;
		$i = 0;
		$stars = null;
		$printedHalf = false;
		while ($i<5) {
			$i++;
			if ($i <= $fulls) {
				$stars .= "<i class='fa fa-star'></i> ";
			} elseif ($half && !$printedHalf) {
				$stars .= "<i class='fa fa-star-half-o'></i> ";
				$printedHalf = true;
			} else {
				$stars .= "<i class='fa fa-star-o'></i> ";
			}
		}
		return $stars;
	}


	/**** update prices and number of total offers  ****/
	function amzUpdatePrices($asin,$expiringHours = 6) {
		$asin = trim($asin);
		$itemData = new stdClass();
		$xmlError = null;
		if (!$asin) return false;
		$sql = "select * from `" . $this->tablePrefix . "products` where asin = " . $this->dbquote($asin) . " and (last_price_update is null or date_add(last_price_update, interval " . $expiringHours . " hour) <= now()) and imported is not null";
		$res = $this->dbquery($sql);
		if (mysqli_num_rows($res)) {
			$row = mysqli_fetch_object($res);

			$XMLurl = $this->amzGetXMLUrl("prices",array("ASIN" => $asin));

			//CURL Version
	    $ch = curl_init();
	    curl_setopt($ch, CURLOPT_URL, $XMLurl);
	    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	    // get the result of http query
	    $output = curl_exec($ch);
	    curl_close($ch);

	    if (!preg_match('/^</',$output)) {
	      $this->error = "Data of $url not completed loaded.<br />
	        Output:<pre>$output</pre>";
	    } else {
	      $xmlData = simplexml_load_string($output);
	    }

	    // echo "<pre>". print_r($xmlData->Items->Request->Errors,true) . "</pre><hr />";
	    if ($xmlData->Items->Request->Errors) {
	    	$xmlError = "----ERROR ASIN:$ASIN<br /><pre>" . print_r($xmlData->Items,true) . "</pre><hr />\n";
	    }

	    // echo "<pre>" . print_r($xmlData,true) . "</pre>";

	    if (!$xmlError) {

	    	$itemData->price_str_lowest = strval($xmlData->Items->Item->OfferSummary->LowestNewPrice->FormattedPrice);
		    if (isset($xmlData->Items->Item->Offers->Offer->OfferListing)) {
		    	if (isset($xmlData->Items->Item->Offers->Offer->OfferListing->SalePrice)) {
				    $itemData->price_str_offer = strval($xmlData->Items->Item->Offers->Offer->OfferListing->SalePrice->FormattedPrice);
				  } else {
				    $itemData->price_str_offer = strval($xmlData->Items->Item->Offers->Offer->OfferListing->Price->FormattedPrice);  	
			    }
			    $itemData->prime = intval($xmlData->Items->Item->Offers->Offer->OfferListing->IsEligibleForPrime) ? 1 : 0;
		    } else {
			    $itemData->price_str_offer = "";
			    $itemData->prime = 0;
		    }
		    //offers amount - to check if it's possible to add to cart
		    $itemData->offers_total = intval($xmlData->Items->Item->Offers->TotalOffers);

		    // echo "<hr />";
	    	// echo "<pre>" . print_r($itemData,true) . "</pre>";
	    }

		  //let's update the database
		  $sql = "update `" . $this->tablePrefix . "products` set ";
		  if ($itemData->price_str_lowest) {
		  	$sql .= "price_str_lowest = " . $this->dbquote($itemData->price_str_lowest) . ",";
		  }
		  if ($itemData->price_str_offer) {
		  	$sql .= "price_str_offer = " . $this->dbquote($itemData->price_str_offer) . ",";
		  }
		  $sql .= "prime = '" . intval($itemData->prime) . "',
		  	offers_total = '" . intval($itemData->offers_total) . "',
		  	last_price_update = now()
		  	where id = '" . $row->id . "'";
		  $this->dbquery($sql);
		  //return new price data
		  return array($itemData->price_str_lowest, $itemData->price_str_offer, $itemData->prime, $itemData->offers_total);
		}
		return false;
	}



	function amzFirstItemUpdate($asin, $verbose=false, $json=false) {
		$error = $success = false;
		
		if (!$asin) {
			if ($json) {
				$error = "Insufficient data";
			} else {
				return false;
			}
		}

		//already imported????
		//or should be tried to re-import
		if (!$error) {
			$sql = "select id from `" . $this->tablePrefix . "products` where asin = " . $this->dbquote($asin) . " and imported is NULL or title = ''";
			$res = $this->dbquery($sql);
			if (!mysqli_num_rows($res)) {
				if ($json) {
					$error = "Already imported or not found in Store Database";
				} else {
					return false;
				}
			}
		}

		if (!$error) {
			$itemData = $this->amzGetItemData($asin);

			// $error = "<pre>" . print_r($itemData,true). "</pre>";
			// echo "<pre>Y" . print_r($itemData,true) . "</pre>";
			if (!$itemData) {
				$sql = "update `" . $this->tablePrefix . "products` set import_failed = now() where asin = " . $this->dbquote($asin);
				$this->dbquery($sql);
				if ($verbose) echo "$asin - Data cannot be fetched from Amazon<br />\n";
				if ($json) {
					$error = "Data cannot be fetched from Amazon";
				} else {
					return false;
				}
			} 
			// elseif (!$itemData->title) {
			// 	if ($json) {
			// 		$error = "Got incomplete data from Amazon";
			// 	} else {
			// 		return false;
			// 	}
			// }
		}


		if (!$error) {

			$sql = "update `" . $this->tablePrefix . "products` set
				parent_asin = " . $this->dbquote($itemData->parent_asin) . ",
				ean = " . $this->dbquote($itemData->ean) . ",
				title = " . $this->dbquote($itemData->title) . ",
				item_url = " . $this->dbquote($itemData->item_url) . ",
				wishlist_url = " . $this->dbquote($itemData->wishlist_url) . ",
				reviews_external_url = " . $this->dbquote($itemData->reviews_external_url) . ",
				reviews_iframe_url = " . $this->dbquote($itemData->reviews_iframe_url) . ",
				reviews_iframe_url_last_update = now(),
				has_reviews = '" . intval($itemData->has_reviews) . "',
				features = " . $this->dbquote($itemData->features) . ",
				brand = " . $this->dbquote($itemData->brand) . ",
				product_group = " . $this->dbquote($itemData->product_group) . ",
				actors = " . $this->dbquote($itemData->actors) . ",
				release_date = " . ($itemData->release_date ? $this->dbquote($itemData->release_date) : "NULL") . ",
				running_time = '" . intval($itemData->running_time) . "',
				platform = " . $this->dbquote($itemData->platform) . ",
				is_movie = '" . intval($itemData->is_movie) . "',
				image_url = " . $this->dbquote($itemData->image_url) . ",
				price_str_lowest = " . $this->dbquote($itemData->price_str_lowest) . ",
				price_str_offer = " . $this->dbquote($itemData->price_str_offer) . ",
				offers_total = '" . intval($itemData->offers_total) . "',
				prime = '" . intval($itemData->prime) . "',
				variations_total = '" . intval($itemData->variations_total) . "',
				variation_dimensions = " . $this->dbquote($itemData->variation_dimensions) . ",
				variations_price_str_lowest = " . $this->dbquote($itemData->variations_price_str_lowest) . ",
				variations_price_str_highest = " . $this->dbquote($itemData->variations_price_str_highest) . ",
				imported = now()
				where asin = " . $this->dbquote($asin);
			$this->dbquery($sql);
			if (mysqli_affected_rows($this->db)) {
				if ($verbose) echo "$asin imported<br />\n";

				//add variations if update succeeded
				if ($itemData->variations) {
					foreach ($itemData->variations as $v_asin => $v_row) {
						$add2sql = array();
						$i = 0;
						//precheck
						$sql = "select asin from `" . $this->tablePrefix . "variations` where asin = " . $this->dbquote($v_asin) . " and parent_asin = " . $this->dbquote($asin);
						$res = $this->dbquery($sql);
						if (!mysqli_num_rows($res)) {
							foreach ($v_row["variationAttributes"] as $attrData) {
								if (++$i > 10) break;
								$add2sql[] = "attr" . $i . "_name = " . $this->dbquote($attrData["Name"]);
								$add2sql[] = "attr" . $i . "_value = " . $this->dbquote($attrData["Value"]);
							}
							$sql = "insert into `" . $this->tablePrefix . "variations` set 
								asin = " . $this->dbquote($v_asin) . ",
								parent_asin = " . $this->dbquote($asin) . ",
								title = " . $this->dbquote($v_row["title"]) . ",
								image_url = " . $this->dbquote($v_row["image_url"]) .
								($add2sql ? ", " . implode(", ", $add2sql) : "") . ",
								created = now()";
							$this->dbquery($sql);
						}
					}
					//import asins to products
					if ($this->handle_variations) {
						$this->importASINlist(implode("\n",array_keys($itemData->variations)),false,"as_variation");
					}
				}
				$success = true;
			} else {
				$sql = "update `" . $this->tablePrefix . "products` set import_failed = now() where asin = " . $this->dbquote($asin);
				$this->dbquery($sql);
				if ($verbose) echo "$asin NOT imported<br />\n";
				if ($json) {
					$error = "Database importing troubles.";
				} else {
					return false;
				}
			}
		}
		//everything went right - let's go on
		
		//editor reviews
		if ($itemData->editorial_reviews) {
			//delete any existing reviews assigend to asin to prevent from dupes
			$sql = "delete from `" . $this->tablePrefix . "editorial_reviews` where asin = " . $this->dbquote($asin);
			$this->dbquery($sql);

			$editorial_reviews = array();					
			if ($this->is_assocArray($itemData->editorial_reviews)) {
				$editorial_reviews[] = (object) array("Content" => $itemData->editorial_reviews["Content"],
																						"Source" => $itemData->editorial_reviews["Source"]);
			} else {
				$pos = 1;
				$editorial_reviews = $itemData->editorial_reviews;
			}
	   
			$pos = 1;
	    foreach ($editorial_reviews as $review) { 
	    	if (!$review->Content) continue;
	    	$sql = "insert into `" . $this->tablePrefix . "editorial_reviews` set
	    		asin = " . $this->dbquote($asin) . ",
	    		source = " . $this->dbquote($review->Source) . ",
	    		review = " . $this->dbquote($review->Content) . ",
	    		pos = '" . $pos++ . "',
	    		created = now()";
				$this->dbquery($sql);
	    }
	   }
    //assigned images
		if ($itemData->images) {
			//delete any existing images assigend to asin to prevent from dupes
			$sql = "delete from `" . $this->tablePrefix . "image_sets` where asin = " . $this->dbquote($asin);
			$this->dbquery($sql);
			//editor reviews
			$pos = 1;
	    foreach ($itemData->images as $image_url) { 
	    	$sql = "insert into `" . $this->tablePrefix . "image_sets` set
	    		asin = " . $this->dbquote($asin) . ",
	    		image_url = " . $this->dbquote($image_url) . ",
	    		pos = '" . $pos++ . "',
	    		created = now()";
				$this->dbquery($sql);
	    }
	  }


	  if ($json) {
	  	$itemImgSrc = null;
	  	if ($itemData->images) {
	  		$itemImgSrc = $this->getImgSizeUrl($itemData->images[0],"xs");
	  	}

			$this->json = json_encode(array(
				"success" => $success, "error" => $error, 
				"asin" => $asin,
				"itemTitle" => $itemData->title,
				"itemImgSrc" => $itemImgSrc,
				"itemAmzLink" => $itemData->item_url,
				"itemStoreUrl" => $this->instDir . "/itm/" . $asin . "/" . $this->tidyup($itemData->title) . ".html"
				));
	  }

	  //variations of parent asin ....
	  if ($this->handle_variations && $itemData->parent_asin && $asin != $itemData->parent_asin) {
	  	//parent already imported?
			$sql = "select id from `" . $this->tablePrefix . "products` where asin = " . $this->dbquote($itemData->parent_asin);
			$res = $this->dbquery($sql);
			if (!mysqli_fetch_object($res) && $this->importASINlist($itemData->parent_asin,false,"variation_parent")) {
				$this->amzFirstItemUpdate($itemData->parent_asin);
			}

	  }

		return true;
	}


	//urls with unique keys will transformed into general ones with key placeholders
	function amzKeysToPlaceholders($str) {
    $str = preg_replace("/" . $this->config->amz_tag . "/", "[ASSOCIATE_TAG]", $str);
    $str = preg_replace("/" . $this->config->amz_access_key . "/", "[ACCESS_KEY_ID]", $str);
    return $str;
	}
	//replace placeholders of urls with unique amazon keys
	function placeholdersToAmzKeys($str) {
    $str = preg_replace("/\[ASSOCIATE_TAG\]/",$this->config->amz_tag, $str);
    $str = preg_replace("/\[ACCESS_KEY_ID\]/",$this->config->amz_access_key, $str);
    return $str;
	}


	function amzGetSuggestions($asin,$link_amz_suggestions_to_amz) {
		$asin = trim($asin);
		$itemData = new stdClass();
		$suggestions = array();
		//xml-URL
		$XMLurl = $this->amzGetXMLUrl("suggestions",array("ASIN" => $asin));

		//CURL Version
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $XMLurl);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    // get the result of http query
    $output = curl_exec($ch);
    curl_close($ch);
    
    if (!preg_match('/^</',$output)) {
      $this->error = "Data of $url not completed loaded.<br />
        Output:<pre>$output</pre>";
    } else {
      $xmlData = simplexml_load_string($output);
    }

    $items = (array)$xmlData->Items;
    // echo "<pre>" . print_r($items["Item"],true) . "</pre>";
    if (isset($items["Item"])) {
	    foreach ($items["Item"] as $item) {
	    	if (!$item->ItemAttributes->Title) continue;
	    	$suggestion["asin"] = strval($item->ASIN);
	    	$suggestion["item_url"] = strval($item->DetailPageURL);
	    	$suggestion["image_url"] = strval($item->LargeImage->URL);
	    	$suggestion["image_url_medium"] = preg_replace('/\.jpg$/',"._SL200_.jpg",strval($item->LargeImage->URL));
	    	$suggestion["title"] = strval($item->ItemAttributes->Title);
	    	$suggestion["lowest_new_price"] = strval($item->OfferSummary->LowestNewPrice->FormattedPrice);

	    	$suggestions[] = (object) $suggestion;

	      //add to store db - if adding is enabled by internal item linking
	      if (!$link_amz_suggestions_to_amz) {
	      	$this->importASINlist($suggestion["asin"],false,"amazon-suggestions");
	      }
	    }
	  }
    return $suggestions;
}



	function amzGetItemData($asin,$verbose=true) {
		if (!$asin) return false;
		$xmlError = false;
		$itemData = new stdClass();
		$this->xmlErrorDataFileContent = null;
		// xml-URL
		$XMLurl = $this->amzGetXMLUrl("ASIN",array("ASIN" => $asin));

		// $XMLurl = "http://www.digidice.eu/error1.xml";

		//CURL Version
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $XMLurl);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    // get the result of http query
    $output = curl_exec($ch);
    curl_close($ch);

		// $output = file_get_contents($this->xmlCacheDir . "webservices.amazon.de/B01DMBT63U.xml");

		// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		// UNCOMMENT CURL AND FWRITE!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    
    if (!preg_match('/^</',$output)) {
      $this->error = "Data of $url not completed loaded.<br />
        Output:<pre>$output</pre>";
    } else {
      $xmlData = simplexml_load_string($output);
    }

    // echo "<pre>". print_r($xmlData->Items->Request->Errors,true) . "</pre><hr />";
    if ($xmlData->Items->Request->Errors) {
    	if ($verbose) {
    		echo "----ERROR ASIN:$ASIN<br /><pre>" . print_r($xmlData->Items,true) . "</pre><hr />\n";
    	} else {
    		$this->xmlErrorDataFileContent = $xmlData->Items;
    	}
			$xmlError = true;
    }



    //save to cacheDir ....
    //create marketplace directory if not exists
    $target = $this->xmlCacheDir . $this->config->amz_marketplace . "/";
    if (!is_dir($target)) {
    	mkdir($target, 0757); 
    }
    $fname = $target . $asin .  ($xmlError ? "_err" : "") . ".xml";
    $fp = fopen($fname, 'w');
		fwrite($fp, $output);
		fclose($fp);
    chmod($fname, 0646); 

    if ($xmlError) {
    	return false;
    }

    // echo "<pre>". print_r($xmlData,true) . "</pre><hr />";

    $wishlist_url = $add2cart_url = $reviews_url = $item_url = false;
    if ($xmlData->Items->Item->ItemLinks[0]) {
	    foreach ( $xmlData->Items->Item->ItemLinks[0] as $tmpData) {
	    	// $tmpData = (object) $tmpData;
	    	// echo "<pre>". print_r($tmpData,true) . "</pre><hr />";
	    	if (preg_match('/wishlist/i',$tmpData->Description)) {
	    		$wishlist_url = strval($tmpData->URL);
	    	} elseif (preg_match('/reviews/i',$tmpData->Description)) {
	    		$reviews_external_url = strval($tmpData->URL);
	    	}
	    }
	  }
    $reviews_iframe_url = $this->amzKeysToPlaceholders(strval($xmlData->Items->Item->CustomerReviews->IFrameURL));
    $reviews_external_url = $this->amzKeysToPlaceholders($reviews_external_url);


    // $add2cart_url = "https://www.amazon.de/gp/aws/cart/add.html?AWSAccessKeyId=[ACCESS_KEY_ID]&amp;AssociateTag=[ASSOCIATE_TAG]&amp;ASIN.1=" . strval($xmlData->Items->Item->ASIN) . "&amp;Quantity.1=1";

    $item_url = strval($xmlData->Items->Item->DetailPageURL);

    $wishlist_url = $this->amzKeysToPlaceholders($wishlist_url);
    // $add2cart_url = $this->amzKeysToPlaceholders($add2cart_url);
    $reviews_url = $this->amzKeysToPlaceholders($reviews_url);
    $item_url = $this->amzKeysToPlaceholders($item_url);


    $editorial_reviews = array();
    foreach ((array) $xmlData->Items->Item->EditorialReviews as $edtReview) {
    	$editorial_reviews = (array) $edtReview;
    }


    $itemData->asin = $asin;
    $itemData->ean = strval($xmlData->Items->Item->ItemAttributes->EAN);
    $itemData->parent_asin = strval($xmlData->Items->Item->ParentASIN);
    $itemData->title = strval($xmlData->Items->Item->ItemAttributes->Title);
    $itemData->item_url = $item_url;
    $itemData->wishlist_url = $wishlist_url;
    // $itemData->add2cart_url = $add2cart_url;
    $itemData->reviews_external_url = $reviews_external_url;
    $itemData->reviews_iframe_url = $reviews_iframe_url;
    $itemData->has_reviews = strval($xmlData->Items->Item->CustomerReviews->HasReviews) == "true" ? 1 : 0;

    $featuresArray = (array) $xmlData->Items->Item->ItemAttributes->Feature;
    if ($featuresArray) {
    	$itemData->features = "<ul><li>" . implode("</li><li>",$featuresArray) . "</li></ul>";
    } else {
    	$itemData->features = null;
    }

    $itemData->brand = strval($xmlData->Items->Item->ItemAttributes->Brand);
    $itemData->product_group = strval($xmlData->Items->Item->ItemAttributes->ProductGroup);

    //variation data
    $itemData->variations_total = 0;
    $itemData->variations = array();
    if (isset($xmlData->Items->Item->Variations->TotalVariations)) {
	    $itemData->variations_total = intval($xmlData->Items->Item->Variations->TotalVariations);
	  	if ($itemData->variations_total && (array) $xmlData->Items->Item->Variations->VariationDimensions->VariationDimension) {
	    	$itemData->variation_dimensions = implode("|",(array) $xmlData->Items->Item->Variations->VariationDimensions->VariationDimension);
	    } else {
	    	$itemData->variation_dimensions = null;
	    }
	    //prices
	    $itemData->variations_price_str_lowest = strval($xmlData->Items->Item->VariationSummary->LowestPrice->FormattedPrice);
	    $itemData->variations_price_str_highest = strval($xmlData->Items->Item->VariationSummary->HighestSalePrice->FormattedPrice);
		
	    //all the vaiations
	    foreach ($xmlData->Items->Item->Variations->Item as $variationData) {
	    	// $variationData = (array) $variationData;
	    	$v_asin = trim(strval($variationData->ASIN));
	    	if (!$v_asin) {
	    		continue;
	    	}
	    	$itemData->variations[$v_asin]["parent_asin"] = strval($variationData->ParentASIN);
	    	$itemData->variations[$v_asin]["image_url"] = strval($variationData->LargeImage->URL);
    		$itemData->variations[$v_asin]["title"] = strval($variationData->ItemAttributes->Title);
	    	$itemData->variations[$v_asin]["variationAttributes"] = array();
	    	foreach ($variationData->VariationAttributes->VariationAttribute as $variationAttribute) {
		    	$itemData->variations[$v_asin]["variationAttributes"][] = (array) $variationAttribute;
	    	}
	    }
		}



    //movie relevant data
    $actors = false;
    if ((array) $xmlData->Items->Item->ItemAttributes->Actor) {
    	$actors = implode("|",(array) $xmlData->Items->Item->ItemAttributes->Actor);
    }
    $itemData->actors = $actors;
    $itemData->release_date = strval($xmlData->Items->Item->ItemAttributes->ReleaseDate);
    $itemData->running_time = strval($xmlData->Items->Item->ItemAttributes->RunningTime);
    $itemData->platform = strval($xmlData->Items->Item->ItemAttributes->Binding);

    $itemData->is_movie = in_array(strtolower($itemData->platform),array("dvd","blu-ray","amazon video")) ? 1 : 0;
    

    $itemData->editorial_reviews = $editorial_reviews;
    $itemData->image_url = strval($xmlData->Items->Item->LargeImage->URL);

    //imagesets
    $itemData->images = array();
    if ($xmlData->Items->Item->ImageSets->ImageSet) {
    	foreach ($xmlData->Items->Item->ImageSets->ImageSet as $imgSetData) {
    		$itemData->images[] = strval($imgSetData->LargeImage->URL[0]);    		
    	}
    }

    $itemData->brand = strval($xmlData->Items->Item->ItemAttributes->Brand);
    $itemData->product_group = strval($xmlData->Items->Item->ItemAttributes->ProductGroup);


    $itemData->price_str_lowest = strval($xmlData->Items->Item->OfferSummary->LowestNewPrice->FormattedPrice);
    if (isset($xmlData->Items->Item->Offers->Offer->OfferListing)) {
    	if (isset($xmlData->Items->Item->Offers->Offer->OfferListing->SalePrice)) {
		    $itemData->price_str_offer = strval($xmlData->Items->Item->Offers->Offer->OfferListing->SalePrice->FormattedPrice);
		  } else {
		    $itemData->price_str_offer = strval($xmlData->Items->Item->Offers->Offer->OfferListing->Price->FormattedPrice);  	
	    }
	    $itemData->prime = intval($xmlData->Items->Item->Offers->Offer->OfferListing->IsEligibleForPrime) ? 1 : 0;
    } else {
	    $itemData->price_str_offer = "";
	    $itemData->prime = 0;
    }

    //offers amount - to check if it's possible to add to cart
    $itemData->offers_total = intval($xmlData->Items->Item->Offers->TotalOffers);

    return $itemData;
	}





	//create api amazon URLs
	//type string to get url for different purposes
	//data array with needed data depending on type given 
	function amzGetXMLUrl($type,$data) {
		$params = array(
			"Service" => "AWSECommerceService",
			"AWSAccessKeyId" => $this->config->amz_access_key,
			"AssociateTag" => $this->config->amz_tag,
			"Timestamp" => gmdate('Y-m-d\TH:i:s\Z')
			);
		$uri = "/onca/xml";

		switch ($type) {
			case 'ASIN':
				$params = array_merge($params, array(
			    "Operation" => "ItemLookup",
			    "ItemId" => $data["ASIN"],
			    "IdType" => "ASIN",
			    "ResponseGroup" => "Large,Variations,VariationImages",

				));
				break;
			case 'keywords':
				$params = array_merge($params, array(
			    "Operation" => "ItemSearch",
		    	"Keywords" => $data["srch"],
			    "ResponseGroup" => "Images,Small",
  				"SearchIndex" => $data["searchindex"],
				));
				break;
			case 'reviews':
				$params = array_merge($params, array(
			    "Operation" => "ItemLookup",
			    "ItemId" => $data["ASIN"],
			    "IdType" => "ASIN",
			    "ResponseGroup" => "Reviews"
				));
				break;
			case 'prices':
				$params = array_merge($params, array(
			    "Operation" => "ItemLookup",
			    "ItemId" => $data["ASIN"],
			    "IdType" => "ASIN",
			    "ResponseGroup" => "Offers"
				));
				break;
			case 'createCart':
				$params = array_merge($params, array(
			    "Operation" => "CartCreate",
			    "Item.1.ASIN" => $data["ASIN"],
			    "Item.1.Quantity" => $data["quantity"],
			    "ResponseGroup" => "Cart"
				));
				break;
			case 'add2cart':
				$params = array_merge($params, array(
			    "Operation" => "CartAdd",
			    "Item.1.ASIN" => $data["ASIN"],
			    "Item.1.Quantity" => $data["quantity"],
			    "CartId" => $data["CartId"],
			    "HMAC" => $data["HMAC"],
			    "ResponseGroup" => "Cart"
				));
				break;
			case 'getCart':
				$params = array_merge($params, array(
			    "Operation" => "CartGet",
			    "CartId" => $data["CartId"],
			    "HMAC" => $data["HMAC"],
			    "ResponseGroup" => "Cart"
				));
				break;
			case 'removeFromCart':
				$params = array_merge($params, array(
			    "Operation" => "CartModify",
			    "CartId" => $data["CartId"],
			    "HMAC" => $data["HMAC"],
			    "Item.1.CartItemId" => $data["cartItemId"],
			    "Item.1.Quantity" => 0,
			    "ResponseGroup" => "Cart"
				));
				break;
			case 'emptyCart':
				$params = array_merge($params, array(
			    "Operation" => "CartClear",
			    "CartId" => $data["CartId"],
			    "HMAC" => $data["HMAC"],
			    "ResponseGroup" => "Cart"
				));
				break;
			case 'modifyCartItem':
				$params = array_merge($params, array(
			    "Operation" => "CartModify",
			    "CartId" => $data["CartId"],
			    "HMAC" => $data["HMAC"],
			    "Item.1.CartItemId" => $data["cartItemId"],
			    "Item.1.Quantity" => $data["quantity"],
			    "ResponseGroup" => "Cart"
				));
				break;
			case 'suggestions':
				$params = array_merge($params, array(
			    "Operation" => "SimilarityLookup",
			    "ItemId" => $data["ASIN"],
			    "ResponseGroup" => "Images,Offers,Small"
				));
				break;
			case 'apiCheck':
				$params = array_merge($params, array(
		    	"Operation" => "ItemSearch",
	    		"Keywords" => "dvd",
					"SearchIndex" => "All",
				));
				$params["AWSAccessKeyId"] = $data["amz_access_key"];
				$params["AssociateTag"] = $data["amz_tag"];
				$this->config->amz_secret_key = $data["amz_secret_key"];
				$this->config->amz_marketplace = $data["amz_marketplace"];
				break;
		}


		// Sort the parameters by key
		ksort($params);
		$pairs = array();
		foreach ($params as $key => $val) {
			$pairs[] = rawurlencode($key) . "=" . rawurlencode($val);
		}
		// Generate the canonical query
		$canonical_query_string = join("&", $pairs);

		// Generate the string to be signed
		$string_to_sign = "GET\n" . $this->config->amz_marketplace . "\n" . $uri . "\n" . $canonical_query_string;

		// Generate the signature required by the Product Advertising API
		$signature = base64_encode(hash_hmac("sha256", $string_to_sign, $this->config->amz_secret_key, true));

		// Generate the signed URL
		$request_url = 'http://' . $this->config->amz_marketplace . $uri . '?' . $canonical_query_string . '&Signature=' . rawurlencode($signature);

		return $request_url;


	}
	/*** admin stuff [-] ***/


}


?>