/***************
 * zStore default layout JS 1.16
 * coming with zStore 1.16
 *  January 17th 2019
 *  https://codecanyon.net/item/zstore-an-amazon-affiliate-store/22713029?ref=themac
 ***************
 *  0. Misc
 *  0.1 finding/defining bootstrap breakpoints
 *  0.2 in_array function definition (not needed now)
 *  1. Masonry
 *  1.1 for product listings (on category pages)
 *      clickable cards
 *  1.2 for category listings (on home page)
 *      clickable cards
 *  2. Footer positioning
 *  3. Sticky Nav (search bar)
 *  4. Slick Sliders
 *  4.1 for product thumbs on product pages
 *  4.2 for amazon item listings on search results pages or amazon suggestions on product pages
 *  4.3 for shop group item listings on product pages
************/


jQuery(document).ready(function() {


/****
  0. Misc [+]
****/ 

// 0.1 finding/defining bootstrap breakpoints
var currentBreakpoint;
// bootstrap breakpoint check-s
function setCurrentBreakpoint() {
  var breakpoints = { 'xs': 'd-block d-sm-none', 
                      'sm': 'd-none d-sm-block d-md-none',
                      'md': 'd-none d-md-block d-lg-none', 
                      'lg': 'd-none d-lg-block d-xl-none',
                      'xl': 'd-none d-xl-block'};
  var dom = $('<div>');
  $(dom).appendTo($('body'));
  $.each(breakpoints, function(breakpoint,cssClass) {
    var bpdom = $('<div>');
    $(bpdom).appendTo($(dom)).text(breakpoint).addClass(cssClass);
    if ($(bpdom).is(":visible")) {
      // $("bpdom").appendTo($(dom)).text(breakpoint).addClass(cssClass);
      currentBreakpoint = breakpoint;
    }
  });
  $(dom).remove();
}
setCurrentBreakpoint();
// $(window).resize(function(){
//   setCurrentBreakpoint();
// });

// 0.2 in_array function definition (not needed now)
function in_array(needle,arr) {
  for(i=0;i<arr.length;i++) {
    if (needle == arr[i]) {
      return true;
    }
  }
  return false;
}

/****
  0. Misc [-]
****/ 

/****
  1. Masonry [+]
****/ 

  /** 1.1 Masonry for product listings (on category pages) **/
  if ($(".row-product-cards").length) {
    //category cards
    var grid = $(".row-cards.masonry-this");
    if (grid.length) {
      grid.imagesLoaded( function() {
        grid.masonry({
          itemSelector: '.card-holder',
          columnWidth: '.card-holder'
        });
      });
    }
    // make whole cards clickable
    if ($(".row-cards").length) {
      $(".row-cards .card a").each(function() {
        var thisLink = $(this).attr("href");
        var thisCard = $(this).closest(".card");
        $(thisCard)
          .addClass("pointer").click(function(e) {
            location.href=thisLink;
            e.preventDefault();
          })
      });
    }
  }


  /** 1.2 Masonry for category listings (on home page) **/
  if ($(".row-category-cards").length) {
    //category cards
    var grid = $(".row-cards.masonry-this");
    if (grid.length) {
      grid.imagesLoaded( function() {
        grid.masonry({
          itemSelector: '.card-holder',
          columnWidth: '.card-holder'
        });
      });
    }
     // make whole cards clickable
    if ($(".row-cards").length) {
      $(".row-cards .card a").each(function() {
        var thisLink = $(this).attr("href");
        var thisCard = $(this).closest(".card");
        $(thisCard)
          .addClass("pointer").click(function(e) {
            location.href=thisLink;
            e.preventDefault();
          })
      });
    }
  }
/****
  1. Masonry [-]
****/ 



/****
  2. Footer positioning [+]
****/ 
  //move footer to the bottom if needed 
  var reposFooter = function() {    
    if($(window).height()-100 > $('body').height()) {
      $("footer").css("margin-top",$(window).height() - $('body').height()-$("footer").height());
    }
  };
  reposFooter();
/****
  2. Footer positioning [-]
****/


/****
  3. Sticky Nav (searchbar) [+]
****/
  // Scroll behaviour
  // on nav bar
  $(window).scroll(function(){
    if ($(window).scrollTop() >= $('.navbar-zstore').height()) {
      $('.navbar-zstore').addClass('fixed-top');
      if ($(window).width() < 560) {
        $('.navbar-zstore .js-col-logo, .navbar-zstore .zstore-xs-top-nav').slideUp();
      }
    }
    else {
      $('.navbar-zstore').removeClass('fixed-top');
      if ($(window).width() < 560) {
        $('.navbar-zstore .js-col-logo, .navbar-zstore .zstore-xs-top-nav').slideDown();
      }
    }
  });
/****
  3. Sticky Nav (searchbar) [-]
****/



/****
  4. Slick Sliders [+]
****/

  /** 4.1 for product thumbs on product pages **/
  var thumbnailWrapper = $('.item-thumbnails-wrapper');
  if (thumbnailWrapper.length) {
    var thumbsCount = $(".item-thumbnails-wrapper .item-thumbnail").length;
    var slickThumbs = $(thumbnailWrapper).slick({
      infinite: false,
      dots: false,
      lazyLoad: 'ondemand',
      slidesToShow: 4,
      nextArrow: '<i class="fa fa-chevron-right slick-next pointer"></i>',
      prevArrow: '<i class="fa fa-chevron-left slick-prev pointer"></i>',
      appendArrows: ".slick-thumbs-arrow-holder"
    });

    $(".slick-thumbs-arrow-holder .slick-prev").hide();

    slickThumbs.on('afterChange', function(event, slick, currentSlide) {    
      var slickToShow = $(thumbnailWrapper).slick('slickGetOption','slidesToShow');
      if (currentSlide == 0) {
        $('.slick-thumbs-arrow-holder .slick-prev').hide();
      } else {
        $('.slick-thumbs-arrow-holder .slick-prev').show();
      }
      

      if (currentSlide + slickToShow >= thumbsCount) {
        $('.slick-thumbs-arrow-holder .slick-next').hide();
      } else {
        $('.slick-thumbs-arrow-holder .slick-next').show();
      }
    });

    $(".item-thumbnails-wrapper .item-thumbnail img").click(function() {
      var image_url = $(this).data("image-url");
      $(".item-image-big-bgholder").css("background-image","url(" + image_url + ")");
      return false;
    });
  }

  /** 4.2 for amazon item listings on search results pages or amazon suggestions on product pages **/
  var suggestionsRow = $(".row-suggestions-amazon");
  if (suggestionsRow.length) {
    var slickArrowHolder = ".slick-suggestions-arrow-holder";
    var slickSuggestions = $(suggestionsRow).slick({
      dots: false,
      infinite: false,
      speed: 300,
      slidesToShow: 4,
      slidesToScroll: 4,
      nextArrow: '<i class="fa fa-chevron-right slick-next pointer"></i>',
      prevArrow: '<i class="fa fa-chevron-left slick-prev pointer"></i>',
      appendArrows: slickArrowHolder,
      responsive: [
        {
          breakpoint: 1024,
          settings: {
            slidesToShow: 3,
            slidesToScroll: 3
          }
        },
        {
          breakpoint: 600,
          settings: {
            slidesToShow: 2,
            slidesToScroll: 2
          }
        },
        {
          breakpoint: 480,
          settings: {
            slidesToShow: 1,
            slidesToScroll: 1
          }
        }
      ]
    });

    $(slickArrowHolder + " .slick-prev").hide();

    slickSuggestions.on('afterChange', function(event, slick, currentSlide) {    
      var slickToShow = $(suggestionsRow).slick('slickGetOption','slidesToShow');
        if (currentSlide === 0) {
          $(slickArrowHolder + ' .slick-prev').hide();
          $(slickArrowHolder + ' .slick-next').show();
        } else {
          $(slickArrowHolder + ' .slick-prev').show();
        }
        
        if (currentSlide + slickToShow >= slick.slideCount) {
          $(slickArrowHolder + ' .slick-next').hide();
        } else {
          $(slickArrowHolder + ' .slick-next').show();
        }
      });
  }

  /** 4.3 for shop group item listings on product pages **/
  var suggestionsGroupRow = $(".row-suggestions-group");
  if (suggestionsGroupRow.length
    && (
      ($(".suggestion-item",suggestionsGroupRow).length > 4 && in_array(currentBreakpoint,["xl","lg"])) ||
      ($(".suggestion-item",suggestionsGroupRow).length > 3 && in_array(currentBreakpoint,["md","sm"])) ||
      ($(".suggestion-item",suggestionsGroupRow).length > 2 && currentBreakpoint == "xs")
    )
  ) {
    var slickArrowHolderG = ".slick-suggestions-group-arrow-holder";
    var slickGroupSuggestions = $(suggestionsGroupRow).slick({
      dots: false,
      infinite: false,
      speed: 300,
      slidesToShow: 4,
      slidesToScroll: 4,
      nextArrow: '<i class="fa fa-chevron-right slick-next pointer"></i>',
      prevArrow: '<i class="fa fa-chevron-left slick-prev pointer"></i>',
      appendArrows: slickArrowHolderG,
      responsive: [
        {
          breakpoint: 1024,
          settings: {
            slidesToShow: 3,
            slidesToScroll: 3,
          }
        },
        {
          breakpoint: 600,
          settings: {
            slidesToShow: 2,
            slidesToScroll: 2
          }
        },
        {
          breakpoint: 480,
          settings: {
            slidesToShow: 1,
            slidesToScroll: 1
          }
        }
      ]
    });


    $(slickArrowHolderG + " .slick-prev").hide();

      slickGroupSuggestions.on('afterChange', function(event, slick, currentSlide) {    
        var slickToShow = $(suggestionsRow).slick('slickGetOption','slidesToShow');
          if (currentSlide === 0) {
            $(slickArrowHolderG + ' .slick-prev').hide();
            $(slickArrowHolderG + ' .slick-next').show();
          } else {
            $(slickArrowHolderG + ' .slick-prev').show();
          }
          
          if (currentSlide + slickToShow >= slick.slideCount) {
            $(slickArrowHolderG + ' .slick-next').hide();
          } else {
            $(slickArrowHolderG + ' .slick-next').show();
          }
        });

    }
/****
  4. Slick Sliders [-]
****/


});